! $Id: ncwrf_to_native.f90,v 1.3 2006/03/21 20:43:06 trn Exp $
! quick and dirty program to read in netcdf wrf input file, and output
! selected fields to native format binary files
! Limitations:
!    No header information (GrADS or native) is generated
!    Output fields are hardwired
!                                
! Written by T. Nehrkorn September 2005
!
! Based on stats_wrf_nc
!
!=================================Make Executable============================
!  Make executable:
! #usually:
!  setenv NETCDF /usr/local/netcdf 
! #on AER linux: /usr/local/netcdf/pgi
!    DEC Alpha
!      f90 ncwrf_to_native.f90 -L$NETCDF/lib -lnetcdf -lm  \
!      -I$NETCDF/include  -free  -o ncwrf_to_native
!
!   linux flags
!      pgf90 ncwrf_to_native.f90 -byteswapio -L$NETCDF/lib -lnetcdf -lm  \
!      -I$NETCDF/include  -Mfree  -o ncwrf_to_native
!
!   Sun flags
!      f90 ncwrf_to_native.f90 -L$NETCDF/lib -lnetcdf -lm  \
!      -I$NETCDF/include  -free  -o ncwrf_to_native
!
!   SGI flags
!      f90 ncwrf_to_native.f90 -L$NETCDF/lib -lnetcdf -lm  \
!      -I$NETCDF/include  -freeform  -o ncwrf_to_native
!
!   IBM flags
!      xlf ncwrf_to_native.f90 -L/usr/local/lib32/r4i4 -lnetcdf -lm  \
!      -I/usr/local/include  '-qFREE(F90)' -o ncwrf_to_native
!
!   If you extra compile flags for other computers - please send along
!
!=================================Run Program================================
!  Run program:
!      ncwrf_to_native   ncwrf_filename  [-options] 
!      options : [-help] [-n native_file]
!  Cannot use any of the options together

!=================================Options====================================
! see subroutine help_info
!

  program ncwrf_to_native

  implicit none
  character (len=256)    :: input_file, native_file
  
! Find out what we need to do first
  call read_args(input_file,native_file)
  print*,"INPUT FILE IS:  ",trim(input_file)
  print*,"Native FILE IS:  ",trim(native_file)
  print*," "

! Now read the file
  call process_file (input_file,native_file) 

end program ncwrf_to_native

!------------------------------------------------------------------------------

  subroutine help_info

  print '(a)'," "
  print '(a)'," Usage: ncwrf_to_native   [-options] ncwrf_file_name "
  print '(a)'," "
  print '(a)'," Current options available are:"
  print '(a)'," Exclusive options (only the last one of these will be used)"
  print '(a)'," -help     : Print this information"                           
  print '(a)'," -n native_file : Provide output filename root"                           

  STOP

  end subroutine help_info

!------------------------------------------------------------------------------

  subroutine read_args(input_file,native_file)

  implicit none
  character (len=256), intent(out)   :: input_file,native_file

  integer :: numarg, i, idummy, nfiles
  integer, external :: iargc
  character (len=256)    :: dummy

  input_file = " "
  native_file = "ncwrf_native"
  numarg = iargc()
  i = 1
  nfiles = 0

  if (numarg == 0) then 
     print *,'No arguments provided'
     call help_info
  endif

  do while (i <= numarg)
    call getarg(i,dummy)

    if (dummy(1:1) == "-") then
      if (dummy(2:2) == "h") then
        print '(2a)',trim(dummy),' option specified'
        if (dummy(3:3) == " " ) call help_info
        if (dummy(2:5) == "help" ) call help_info
      elseif (dummy(2:2) == "n") then
        i = i+1
        call getarg(i,native_file)
        print '(2a)','Native file specified: ',trim(native_file)
      else
        print '(2a)','Invalid option specified: ',trim(dummy)
        call help_info
      endif
    else
       nfiles = nfiles + 1
       if (nfiles .eq. 1) then
          input_file = dummy
       else
          print *,'Too many input files specified'
          call help_info
       endif
    endif

    i = i+1

  enddo

  if (input_file == " ") then
     print *,'No input files specified'
     call help_info
  endif

  end subroutine read_args
!------------------------------------------------------------------------------
  subroutine process_file( file,native_file )
        
    implicit none

  include 'netcdf.inc'

    character (len=256), intent(in) :: file,native_file

    character (len=256) :: temp_file
    character (len=80) :: var_u(1)=(/'U'/), var_v(1)=(/'V'/), var_half(2)=(/'T     ','QVAPOR'/)
    integer, parameter :: type_u=1, type_v=2, type_half=3, ntype=3
    integer :: iu_out(ntype)=(/11,12,13/)
    integer :: nvar(ntype), n_out(ntype), itype, ivar, rec, recl
    character (len=20) :: type_out(ntype)
    logical :: static_fields
    integer :: i,ivtype
  
    character (len=80) :: varnam, att_name, value_chr, print_time
    character (len=80) :: att_sav(2)
    integer :: unit_place=2,order_place=1
    integer, parameter :: maxdims=4
    integer :: dimids(maxdims), dims(maxdims)

    integer cdfid, rcode, id_var, id_att,attlen, cdfid2
    integer nDims, nVars, nAtts, unlimDimID, ndim2
    integer var_type, dims3, id_time, n_times, itimes, iatt

    double precision,  allocatable, dimension(:,:,:) :: data_dp_r
    real,    allocatable, dimension(:,:,:) :: data_r
    integer, allocatable, dimension(:,:,:) :: data_i
    character (len=80) :: times(100)
    integer istart(maxdims), iend(maxdims), middel(maxdims)
    integer istart_t(2), iend_t(2)
    real    sample_value_r
    integer value_int

    nvar = 0
    do i=1,ntype
       if (i .eq. type_u) nvar(i) = size(var_u)
       if (i .eq. type_v) nvar(i) = size(var_v)
       if (i .eq. type_half) nvar(i) = size(var_half)
       if (i .eq. type_u) type_out(i) = 'u_3d'
       if (i .eq. type_v) type_out(i) = 'v_3d'
       if (i .eq. type_half) type_out(i) = 'half_3d'
    enddo
    n_out(:) = 0
    rcode = nf_open(trim(file), NCNOWRIT, cdfid )
    if( rcode == 0) then
       write(*,*) ' '
    else
       write(*,*) ' error opening netcdf file ',trim(file)
       stop
    end if

    ! Get the times first:

    rcode = nf_inq_varid ( cdfid, 'Times', id_var )
    if (rcode .ne. 0) then
       print*,"This looks like a static file"
       print*," "
       static_fields = .TRUE.
       n_times=1
    else
       id_time = ncvid( cdfid, 'Times', rcode )
       static_fields = .FALSE.
    endif

    if (.not. static_fields) then
       rcode = nf_inq_var( cdfid, id_time, varnam, ivtype, ndims, dimids, natts )
       if (ndims .gt. maxdims) then
          print *,'Warning: ndims= ',ndims,' > maxdims= ',maxdims
          ndims = maxdims
       endif
       do i=1,ndims
          rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
       enddo

       times = ' '
       n_times = dims(2)
       do i=1,dims(2)
          istart_t(1) = 1
          iend_t(1) = dims(1)
          istart_t(2) = i
          iend_t(2) = 1
          rcode = NF_GET_VARA_TEXT  ( cdfid, id_time,  &
               istart_t, iend_t,    &
               times(i)          )
       enddo

       print*,"TIMES in file"
       do itimes = 1,n_times
          print_time = times(itimes)
          print '(i8,a)',itimes,trim(print_time)
       enddo

    endif !.not. static_fields

    rcode = nf_inq(cdfid, nDims, nVars, nAtts, unlimDimID)

       TIME_LOOP: do itimes = 1,n_times
          print*,"  "
          if (.not. static_fields) then
             print_time = times(itimes)
             print*,"TIME: ",trim(print_time)
          endif
          VAR_LOOP: do id_var = 1,nVars

             dims = 1
             rcode = nf_inq_var( cdfid, id_var, varnam, ivtype, nDims, dimids, nAtts )
             var_type = ivtype

             do i=1,ndims
                rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
             enddo

             att_sav(:)=' '
             do id_att = 1,nAtts
                rcode = nf_inq_attname( cdfid,id_var,id_att,att_name )
                rcode = nf_inq_att( cdfid,id_var,att_name,ivtype,attlen )
                if ( ivtype == nclong ) then
                   rcode = NF_GET_ATT_INT(cdfid, id_var, att_name, value_int )
                endif
                if ( ivtype == ncchar ) then
                   rcode = NF_GET_ATT_TEXT(cdfid, id_var, att_name, value_chr )
                   if (index(value_chr(1:attlen),char(0)) .eq. 0) then
                      if (att_name(1:4) == "unit") &
                           & att_sav(unit_place) = value_chr(1:attlen)
                      if (att_name(1:11) == "MemoryOrder") &
                           & att_sav(order_place) = value_chr(1:attlen)
                   endif
                endif
             enddo

             middel        = 1
             istart        = 1
             iend          = 1
             if (static_fields) then
                ndim2=nDims
             else
                istart(nDims) = itimes
                iend(nDims) = itimes
                ndim2=nDims-1
             endif

             do i = 1,ndim2
                iend(i)     = dims(i)
                middel(i)   = iend(i)/2
                if ( middel(i) .le. 0  .or. middel(i) .gt. dims(i) ) then
                   middel(i) = iend(i)/2
                   if (middel(i) == 0) middel(i) = 1
                endif
             enddo

             itype = 0
             ivar = 0
             if (ndim2 .eq. 3 .and. trim(att_sav(order_place)) .eq. 'XYZ') then
                do i=1,size(var_u)
                   if (trim(varnam) .eq. trim(var_u(i))) then
                      itype=type_u
                      ivar=i
                      exit
                   endif
                enddo
                do i=1,size(var_v)
                   if (trim(varnam) .eq. trim(var_v(i))) then
                      itype=type_v
                      ivar=i
                      exit
                   endif
                enddo
                do i=1,size(var_half)
                   if (trim(varnam) .eq. trim(var_half(i))) then
                      itype=type_half
                      ivar=i
                      exit
                   endif
                enddo
             endif
             if (itype .eq. 0) cycle VAR_LOOP
             if (allocated(data_r)) deallocate (data_r)
             allocate (data_r(iend(1),iend(2),iend(3)))
             if (var_type .eq. ncfloat)  then   !get_real
                call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
             elseif (var_type .eq. ncdouble)  then   !get_real_double
                allocate (data_dp_r(iend(1),iend(2),iend(3)))
                call ncvgt( cdfid,id_var,istart,iend,data_dp_r,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
                data_r = data_dp_r
                deallocate (data_dp_r)
             elseif (var_type .eq. nclong) then   !get_int
                allocate (data_i(iend(1),iend(2),iend(3)))
                call ncvgt( cdfid,id_var,istart,iend,data_i,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
                data_r = data_i
                deallocate (data_i)
             else
                cycle VAR_LOOP !skip non-supported variable types
             endif
             n_out(itype) = n_out(itype) + 1
             if (n_out(itype) .eq. 1) then
                temp_file = trim(native_file) // '_' // trim(type_out(itype)) // '.dat'
                inquire(iolength=recl) data_r
                print '(4a,2i8)', 'Opening ',trim(temp_file), &
                     & ' with type, unit, recl= ',trim(type_out(itype)),iu_out(itype),recl
                open(unit=iu_out(itype),file=trim(temp_file),form='unformatted',access='direct',recl=recl)
             endif
             sample_value_r = data_r(middel(1),middel(2),middel(3))
             rec=ivar + (itimes-1)*nvar(itype)
             write (*,301) 'type:',trim(type_out(itype)),' rec=',rec,&
                  & varnam,nDims-1,trim(att_sav(order_place)), &
                  & iend(1),iend(2),iend(3), &
                  & sample_value_r,trim(att_sav(unit_place))
301    format(3a,i4,1x,A17,"  ",i2,"  ",A3,"  ",3(1x,i4),"  ",G18.10E2,"  ",A)
             write (iu_out(itype),rec=rec) data_r
             if (allocated(data_r)) deallocate (data_r)
          end do VAR_LOOP
       enddo TIME_LOOP

    do i=1,ntype
       print '(a,i4,2a)','Processed ',n_out(i),' for type=',trim(type_out(i))
       if (n_out(i) .gt. 0) close(iu_out(i))
    enddo
    call ncclos(cdfid,rcode)

    print*,"  "
    print*,"   --- End of input file ---   "

  end subroutine process_file
