#!/usr/bin/perl
# $Id: ncwrf_to_native.pl,v 1.3 2006/03/28 20:58:09 trn Exp $

# script for running the ncwrf_to_native program

use strict;
use File::Basename;
use Getopt::Long ;

umask oct("002");
$|=1; #use unbuffered I/O

my %filesuf=('half' => 'half_3d','full' => 'full_3d','u' => 'u_3d','v' => 'v_3d',
	     'sfc' => 'sfc_2d','pcp' => 'pcp_2d', '2m' => '2m_2d', 'uv10m' => 'uv10m_2d', 
	     'soil' => 'soil_3d');
my @varhalf=(13,53);
my @varu=(33);
my @varv=(34);
my %varcodes=('half' => \@varhalf, 'u' => \@varu, 'v' => \@varv);

my ($verbose, $help, $ncwrf, $wrf_native, $native, @types);
$verbose=1;

my %options = (
	       "verbose!"        => \$verbose, 
	       "help|?!"         => \$help,
	       "ncwrf=s"  => \$ncwrf,
	       "wrf_native=s"  => \$wrf_native,
	       "native=s"  => \$native,
	       "types=s@"  => \@types,
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "help|?!"         => "generate help message",
	       "ncwrf=s"  => "[REQ:] Input netcdf WRF filename",
	       "wrf_native=s"  => "[REQ:] native WRF filename root to use as template",
	       "native=s"  => "output native filename root",
	       "types=s@"  => "types to process",
	      );
$help=1 unless GetOptions (%options);
if (!$ncwrf) {
  $help=1; #required
} else {
  my ($native_name,$native_path,$native_suf) = fileparse($ncwrf,'');
  $native=$native_name."_native" unless $native; #default
}
if (!$wrf_native) {
  $help=1; #required
} else {
  $wrf_native=$wrf_native.'_' unless substr($wrf_native,-1) eq '_';
}
$help=1 unless $wrf_native; #required
@types = keys(%varcodes) unless @types; #default
my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
$verbose=1 if $help;
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

my $cmd0=dirname($0);

my $cmd;

$cmd="$cmd0/ncwrf_to_native -n $native $ncwrf";
print "At ",scalar localtime()," Running $cmd\n" if $verbose;
die ("Error running $cmd\n") if system($cmd);

foreach my $type (@types) {
  my $vars=$varcodes{$type};
  foreach my $suf (".hdr",".ctl","_grid.ctl") {
    my $infile=$wrf_native.$filesuf{$type}.$suf;
    if (open (INFILE,"<$infile")) {
      my $outfile=$native.'_'.$filesuf{$type}.$suf;
      if (open (OUTFILE,">$outfile")) {
	my $comment_str = $suf eq ".hdr" ? "" : "*";
	print "Creating $outfile from $infile\n";
	print OUTFILE $comment_str.
	  'File created by $Id: ncwrf_to_native.pl,v 1.3 2006/03/28 20:58:09 trn Exp $'."\n";
	print OUTFILE $comment_str."From input file $infile\n";
	foreach my $line (<INFILE>) {
	  print OUTFILE $line unless ($line =~ /^\s*(vars\s*=|VARS|var|ENDVARS)/);
	  if ($suf ne ".hdr") {
	    print OUTFILE "VARS ".scalar(@$vars)."\n" if ($line =~ /^VARS\s*\d+/);
	    foreach my $var (@$vars) {
	      print OUTFILE $line if ($line =~ /^var[0]*$var\s+/);
	    }
	    print OUTFILE $line if ($line =~ /^ENDVARS/);
	  }
	}
	if ($suf eq ".hdr") {
	  print OUTFILE "vars       =        ".join("\nvars       =        ",@$vars)."\n";
	}
	close (OUTFILE) or warn "Could not close OUTFILE=$outfile\n";
      } else {
	warn "Could not open >$outfile\n";
      }
      close (INFILE) or warn "Could not close <$infile\n";
    } else {
      warn("Could not open <$infile\n");
    }
  }
}

1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script runs ncwrf_to_native
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
print "
NOTE: The GrADS control will only have the correct DSET if the filename (w/o the 
      leading path) used for -native is the same as that for -wrf_native
      (that is the default)
";
die 
"Usage: $0 [options]\n";
}
