#!/usr/bin/perl

#  # $Id: prep_rerun_wrf.pl,v 1.1 2005/09/02 15:18:14 trn Exp $

use strict;

use Getopt::Long ;

my ($verbose, $debug, $help, $oldmoad, $newmoad);
$verbose=1;
my %options = (
                "verbose!"        => \$verbose, 
                "debug!"        => \$debug, 
                "help|?!"         => \$help,
                "oldmoad=s"       => \$oldmoad,
                "newmoad=s"       => \$newmoad
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "debug!"        => "debug mode", 
	       "help|?!"         => "generate help message",
               "oldmoad=s"         => "Required: name of old (existing) MOAD dataroot",
               "newmoad=s"         => 
	       "Required: name of new MOAD dataroot (will be overwritten if it exists)"
	      );
$help=1 unless GetOptions (%options);
$help=1 unless $newmoad; #required
$help=1 unless $newmoad; #required
$verbose = 1 if $debug;
$verbose = 1 if $help;
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

my $run_dir=`pwd`;
chomp $run_dir;
chdir($oldmoad) or die "Could not chdir to oldmoad=$oldmoad\n";
my $full_old=`pwd`;
chomp $full_old;
print "full_old=$full_old\n" if $debug;
chdir($run_dir) or die "Could not chdir back to run_dir=$run_dir\n";
if (! chdir($newmoad)) {
  print "Making new directory, old one did not exist\n" if $debug;
  chdir($run_dir) or die "Could not chdir back to run_dir=$run_dir\n";
  unlink($newmoad) if (-e $newmoad);
  mkdir ($newmoad, 0777) or die "Could not mkdir newmoad=$newmoad\n";
  chdir ($newmoad) or die "Could not chdir to newmoad=$newmoad\n";
}
foreach my $newdir ("siprd","static","log","wrfprd") {
  rmdir("$newdir") if (-d "$newdir");
  unlink("$newdir") if (-e "$newdir");
  if ("$newdir" ne "siprd") {
    mkdir ("$newdir",0777) or die "Could not mkdir $newdir in $newmoad\n";
  } else {
    symlink("$full_old/$newdir","$newdir") or die "Could not symlink $full_old/$newdir to $newmoad\n";
  }
}
if (-r "$full_old/static/mpi_machines.conf"){
  system("/bin/cp $full_old/static/mpi_machines.conf static/mpi_machines.conf") == 0 or
    warn "Could not /bin/cp $full_old/static/mpi_machines.conf static/mpi_machines.conf\n";
}

1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script sets up a MOAD dataroot for a new WRF run (using run_wrf.pl), reusing the input files
 from a previous wrfprep.pl run
 The siprd subdirectory from oldmoad is linked to newmoad; this means the input data files 
 and the namelist.input from oldmoad/siprd will be used by run_wrf.pl in the new run.
 If desired, edit the namelist.input there first before submitting run_wrf.pl.
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
