! $Id: stats_wrf_nc.f90,v 1.9 2007/01/08 20:25:54 trn Exp $
! stripped down version of read_wrf_nc.f with
! added capability: compute diffs between 2 files, or
!                   between times in 1 file
!                                
! Written by T. Nehrkorn March 2005
!
!  Special program to only read netCDF files, and write some 
!  file information on the screen
!  Can read input/output and static files
!
!=================================Make Executable============================
!  Make executable:
!    DEC Alpha
!      f90 stats_wrf_nc.f90 -L/usr/local/netcdf/lib -lnetcdf -lm  \
!      -I/usr/local/netcdf/include  -free  -o stats_wrf_nc
!
!   linux flags
!      pgf90 stats_wrf_nc.f90 -L/usr/local/netcdf/lib -lnetcdf -lm  \
!      -I/usr/local/netcdf/include  -Mfree  -o stats_wrf_nc
!
!   Sun flags
!      f90 stats_wrf_nc.f90 -L/usr/local/netcdf/lib -lnetcdf -lm  \
!      -I/usr/local/netcdf/include  -free  -o stats_wrf_nc
!
!   SGI flags
!      f90 stats_wrf_nc.f90 -L/usr/local/netcdf/lib -lnetcdf -lm  \
!      -I/usr/local/netcdf/include  -freeform  -o stats_wrf_nc
!
!   IBM flags
!      xlf stats_wrf_nc.f90 -L/usr/local/lib32/r4i4 -lnetcdf -lm  \
!      -I/usr/local/include  -qfree=f90 -qsuffix=f=f90 -o stats_wrf_nc
!
!   If you extra compile flags for other computers - please send along
!
!=================================Run Program================================
!  Run program:
!      stats_wrf_nc   wrf_data_file_name  [-options] 
!      options : [-help] [-head] [-m] [-M z] [-s] [-S x y z] 
!                            [-t] [-v VAL] [-V VAL] [-w VAL]
!                            [-EditData]
!  Cannot use any of the options together

!=================================Options====================================
! see subroutine help_info
!
!  May 2004
!  Cindy Bruyere
!

  program stats_wrf_nc

  implicit none
  character (len=256)    :: input_file, input_file2
  character (len=3)     :: option                        
  integer :: plot_dim(3), time_diff
  logical :: by_level

! Find out what we need to do first
  call read_args(input_file,input_file2,option,plot_dim,time_diff,by_level)
  print*,"INPUT FILE IS:  ",trim(input_file)
  if (len_trim(input_file2) .gt. 0) print*,"INPUT FILE 2 IS:",trim(input_file2)
  print*," "


! Now read the file
  call get_info_from_cdf (input_file,input_file2,option,plot_dim,time_diff,by_level) 


end program stats_wrf_nc

!------------------------------------------------------------------------------

  subroutine help_info

  print '(a)'," "
  print '(a)'," Usage: stats_wrf_nc   [-options] wrf_data_file_name wrf_data_file_name2 "
  print '(a)'," "
  print '(a)'," Values of full fields will be printed out for wrf_data_file_name, unless"
  print '(a)'," 1- wrf_data_file_name2 is supplied: compute diffs between 2 files"
  print '(a)'," 2- difftime is specified: compute diffs all times and time difftime"
  print '(a)'," "
  print '(a)'," Current options available are:"
  print '(a)'," Exclusive options (only the last one of these will be used)"
  print '(a)'," -help     : Print this information"                           
  print '(a)'," -head     : Print header information only"                           
  print '(a)'," -s        : Print list of fields available for each time, "
  print '(a)',"             plus stats and a sample value for each field"
  print '(a)',"             Sample value is in middle of domain"
  print '(a)',"             Also print the header information"
  print '(a)',"             Default if no options are given"
  print '(a)'," -S  x y z : Print list of fields available for each time, "
  print '(a)',"             plus stats and a sample value for each field"
  print '(a)',"             Sample value is at point x y z in domain"
  print '(a)',"             Also print the header information"
  print '(a)'," -t        : Print only the times in the file"                  
  print '(a)'," "
  print '(a)'," Additional options"
  print '(a)'," -difftime t1: Compute diffs wrt time period t1"
  print '(a)'," -bylevel    : Compute stats by level"
  

  STOP

  end subroutine help_info

!------------------------------------------------------------------------------

  subroutine read_args(input_file,input_file2,option,plot_dim,time_diff, by_level)

  implicit none
  character (len=256), intent(out)   :: input_file,input_file2
  character (len=3), intent(out)     :: option                        
  integer, intent(out) :: plot_dim(3), time_diff
  logical, intent(out) :: by_level

  integer :: numarg, i, idummy, nfiles
  integer, external :: iargc
  character (len=256)    :: dummy

  input_file = " "
  input_file2 = " "
  nfiles = 0
  option = "-s"
  plot_dim = 0
  time_diff = 0
  by_level = .FALSE.
  numarg = iargc()
  i = 1

  if (numarg == 0) call help_info

  
  do while (i <= numarg)
    call getarg(i,dummy)

    if (dummy(1:1) == "-") then
      if (dummy(2:2) == "h") then
        if (dummy(3:3) == " " ) call help_info
        if (dummy(2:5) == "help" ) call help_info
        if (dummy(2:5) == "head" ) option = "-h"
      elseif (dummy(2:2) == "s") then
        option = dummy
      elseif (dummy(2:2) == "S") then
        option = dummy
        i = i+1
        call getarg(i,dummy)
        read(dummy,*)idummy
        plot_dim(1) = idummy
        i = i+1
        call getarg(i,dummy)
        read(dummy,*)idummy
        plot_dim(2) = idummy
        i = i+1
        call getarg(i,dummy)
        read(dummy,*)idummy
        plot_dim(3) = idummy
      elseif (dummy(2:2) == "t") then
         option = dummy
      elseif(dummy(2:9) == "difftime") then
         if (nfiles .gt. 1) call help_info
         i = i+1
         call getarg(i,dummy)
         read(dummy,*) idummy
         time_diff = idummy
      elseif(dummy(2:8) == "bylevel") then
         by_level=.TRUE.
      else
        call help_info
      endif
    else
       nfiles = nfiles + 1
       if (nfiles .eq. 1) then
          input_file = dummy
       elseif (nfiles .eq. 2) then
          if (time_diff .ne. 0) call help_info
          input_file2 = dummy
       else
          call help_info
       endif
    endif

      i = i+1

  enddo

  if (input_file == " ") call help_info

  end subroutine read_args
!------------------------------------------------------------------------------
  subroutine get_info_from_cdf( file,file2,option,plot_dim,time_diff,by_level )
        
    implicit none

  include 'netcdf.inc'

    character (len=256), intent(in) :: file,file2
    character (len=3),intent(in)    :: option
    integer, intent(in) :: plot_dim(3), time_diff
    logical, intent(in) :: by_level
    
    logical :: static_fields, two_files, mismatch
    integer :: i,j,k,ivtype, length, ivtype2
    integer :: get_x, get_y, get_z
  
    character (len=80) :: varnam, att_name, value_chr, print_time, varnam2
    character (len=80) :: att_sav(2)
    integer :: unit_place=2,order_place=1
    integer, parameter :: maxdims=4
    integer :: dimids(maxdims), dims(maxdims), dimids2(maxdims), dims2(maxdims)

    integer cdfid, rcode, id_var, id_att,attlen, cdfid2, id_var2
    integer nDims, nVars, nAtts, unlimDimID, ndim2, nDims2, nVars2, nAtts2, unlimDimID2
    integer var_type, dims3, id_time, n_times, itimes, iatt

    double precision,  allocatable, dimension(:,:,:,:) :: data_dp_r
    real,    allocatable, dimension(:,:,:,:) :: data_r, data_r2
    integer, allocatable, dimension(:,:,:,:) :: data_i
    character (len=80) :: times(100)
    integer istart(maxdims), iend(maxdims), middel(maxdims)
    integer istart_t(2), iend_t(2)
    real    sample_value_r, minvalue_r, maxvalue_r
    real, allocatable, dimension(:) :: value_real
    integer sample_value_i, minvalue_i, maxvalue_i, value_int

    integer :: len_2d, len_3d, n_stats
    real :: min_stats, max_stats, bias_stats, rms_stats

    rcode = nf_open(trim(file), NCNOWRIT, cdfid )
    if( rcode == 0) then
       write(*,*) ' '
    else
       write(*,*) ' error opening netcdf file ',trim(file)
       stop
    end if
    if (len_trim(file2) .gt. 0) then
       rcode = nf_open(trim(file2), NCNOWRIT, cdfid2 )
       if( rcode == 0) then
          write(*,*) ' '
       else
          write(*,*) ' error opening netcdf file ',trim(file2)
       end if
    else
       rcode=1
    endif
    two_files = rcode .eq. 0

    ! Get the times first:

    rcode = nf_inq_varid ( cdfid, 'Times', id_var )
    if (rcode .ne. 0) then
       print*,"This looks like a static file"
       print*," "
       static_fields = .TRUE.
       n_times=1
       if (time_diff .ne. 0) then
          print *,'difftime option invalid for static files'
          stop 'difftime option invalid for static files'
       endif
       if (option(1:2) .eq. "-t") then
          print *,'-t option invalid for static files'
          stop '-t option invalid for static files'
       endif
    else
       id_time = ncvid( cdfid, 'Times', rcode )
       static_fields = .FALSE.
    endif

    if (.not. static_fields) then
       rcode = nf_inq_var( cdfid, id_time, varnam, ivtype, ndims, dimids, natts )
       if (ndims .gt. maxdims) then
          print *,'Warning: ndims= ',ndims,' > maxdims= ',maxdims
          ndims = maxdims
       endif
       do i=1,ndims
          rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
       enddo

       times = ' '
       n_times = dims(2)
       do i=1,dims(2)
          istart_t(1) = 1
          iend_t(1) = dims(1)
          istart_t(2) = i
          iend_t(2) = 1
          rcode = NF_GET_VARA_TEXT  ( cdfid, id_time,  &
               istart_t, iend_t,    &
               times(i)          )
       enddo

       print*,"TIMES in file"
       do itimes = 1,n_times
          print_time = times(itimes)
          if (time_diff .eq. itimes) then
             print '(i8,2a)',itimes,' Diffs computed from this time: ',trim(print_time)
          else
             print '(i8,a)',itimes,trim(print_time)
          endif
       enddo
       ! If we only want information about time
       if ( option == "-t" ) then
          STOP
       endif

       if (time_diff .gt. n_times) then
          write (*,*) 'Invalid difftime specified: ',time_diff, &
               & ' There are only ',n_times,' times in the file'
          stop 'Invalid difftime specified '
       endif

    endif !.not. static_fields

    rcode = nf_inq(cdfid, nDims, nVars, nAtts, unlimDimID)
    if (two_files) rcode = nf_inq(cdfid2, nDims2, nVars2, nAtts2, unlimDimID2)

    ! Get some header information
    if ( option .ne. "-v" .and. option .ne. "-V" .and.  &
         option .ne. "-w" .and. option .ne. "-n" ) then
       print*,"HEADER INFORMATION:"
       print*," "
       do iatt = 1,nAtts
          rcode = nf_inq_attname(cdfid,ncglobal,iatt,att_name)
          rcode = nf_inq_att( cdfid,ncglobal,att_name,ivtype,attlen )
          if (ivtype == ncchar) then
             rcode = NF_GET_ATT_TEXT(cdfid, ncglobal, att_name, value_chr )
             write(*,'(A," : ",A)') att_name(1:40),value_chr(1:attlen)
          elseif (ivtype == nclong) then
             rcode = NF_GET_ATT_INT(cdfid, ncglobal, att_name, value_int )
             write(*,'(A," : ",i5)') att_name(1:40),value_int
          elseif (ivtype == ncfloat) then
             allocate (value_real(attlen))
             rcode = NF_GET_ATT_REAL(cdfid, ncglobal, att_name, value_real )
             if (attlen .gt. 1) then
                print*,att_name(1:40),": ",value_real
             else
                write(*,'(A," : ",f12.4)') att_name(1:40),value_real(1)
             endif
             deallocate (value_real)
          endif
       enddo
       if ( option == "-h") STOP
       print*," "
       print*,"--------------------------------------------------------------------- "
       print*," "
    endif


    !===========================================================================================
    ! Option -s  :  Write out all field names plus stats and a sample value of each field
    if      ( option == "-s" .or. option == "-S" ) then

       TIME_LOOP: do itimes = 1,n_times
          print*,"  "
          if (.not. static_fields) then
             print_time = times(itimes)
             print*,"TIME: ",trim(print_time)
          endif
          VAR_LOOP: do id_var = 1,nVars

             dims = 1
             rcode = nf_inq_var( cdfid, id_var, varnam, ivtype, nDims, dimids, nAtts )
             var_type = ivtype

             do i=1,ndims
                rcode = nf_inq_dimlen( cdfid, dimids(i), dims(i) )
             enddo
             if (two_files) then
                id_var2=-99
                do i=1,nVars2
                   rcode = nf_inq_var( cdfid2, i, varnam2, ivtype2, nDims2, dimids2, nAtts2 )
                   if (trim(varnam2) .eq. trim(varnam)) then
                      id_var2=i
                      exit
                   end if
                end do
                if (id_var2 .eq. -99) then
                   print '(4a)','Warning: skipping varnam= ',trim(varnam),' because not in 2nd file'
                   cycle VAR_LOOP
                end if
                if (nAtts .ne. nAtts2) print '(a,2i10)','Warning: nAtts2 ne nAtts ',&
                     & nAtts, nAtts2
                mismatch = nDims2 .ne. nDims
                if (.not. mismatch) then
                   do i=1,ndims
                      rcode = nf_inq_dimlen( cdfid2, dimids2(i), dims2(i) )
                      mismatch=mismatch .or. dims(i) .ne. dims2(i)
                   end do
                end if
                if (mismatch) then
                   print '(3a,2i10)','Skipping ',trim(varnam),' because of dim mismatch:ndims= ',&
                        & nDims, nDims2
                   print '(a,10i10)','dim mismatch: dims2,dims= ',&
                        & dims(1:nDims), dims2(1:nDims2)
                   cycle VAR_LOOP
                end if
             end if
             att_sav(:)=' '
             do id_att = 1,nAtts
                rcode = nf_inq_attname( cdfid,id_var,id_att,att_name )
                rcode = nf_inq_att( cdfid,id_var,att_name,ivtype,attlen )
                if ( ivtype == nclong ) then
                   rcode = NF_GET_ATT_INT(cdfid, id_var, att_name, value_int )
                endif
                if ( ivtype == ncchar ) then
                   rcode = NF_GET_ATT_TEXT(cdfid, id_var, att_name, value_chr )
                   if (index(value_chr(1:attlen),char(0)) .eq. 0) then
                      if (att_name(1:4) == "unit") &
                           & att_sav(unit_place) = value_chr(1:attlen)
                      if (att_name(1:11) == "MemoryOrder") &
                           & att_sav(order_place) = value_chr(1:attlen)
                   endif
                endif
             enddo

             middel        = 1
             istart        = 1
             iend          = 1
             if (static_fields) then
                ndim2=nDims
             else
                istart(nDims) = itimes
                iend(nDims) = itimes
                ndim2=nDims-1
             endif

             len_2d = 1
             len_3d = 1
             do i = 1,ndim2
                if (i .le. 2) len_2d = len_2d * dims(i)
                if (i .le. 3) len_3d = len_3d * dims(i)
                iend(i)     = dims(i)
                middel(i)   = iend(i)/2
                if (option .eq. "-S" .and. i .le. 3) middel(i) = plot_dim(i)
                if ( middel(i) .le. 0  .or. middel(i) .gt. dims(i) ) then
                   middel(i) = iend(i)/2
                   if (middel(i) == 0) middel(i) = 1
                endif
             enddo

             if (allocated(data_r)) deallocate (data_r)
             if (allocated(data_r2)) deallocate (data_r2)
             allocate (data_r(istart(1):iend(1),istart(2):iend(2),istart(3):iend(3), &
                  & istart(4):iend(4)))
             if (time_diff .gt. 0 .or. two_files) &
                  & allocate (data_r2(istart(1):iend(1),istart(2):iend(2),istart(3):iend(3), &
                  & istart(4):iend(4)))
             if (var_type .eq. ncfloat)  then   !get_real
                call ncvgt( cdfid,id_var,istart,iend,data_r,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
                if (time_diff .gt. 0) then
                   istart(ndims) = time_diff
                   iend(ndims) = time_diff
                   call ncvgt( cdfid,id_var,istart,iend,data_r2,rcode)
                   if (rcode .ne. 0) then
                      print *,'Error reading time_diff= ',time_diff,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                endif
                if (two_files) then
                   if (ivtype2 .eq. ncdouble) then
                      allocate (data_dp_r(istart(1):iend(1),istart(2):iend(2),istart(3):iend(3), &
                           & istart(4):iend(4)))
                      call ncvgt( cdfid2,id_var2,istart,iend,data_dp_r,rcode)
                      data_r2=data_dp_r
                      deallocate(data_dp_r)
                   elseif (ivtype2 .eq. ncfloat) then
                      call ncvgt( cdfid2,id_var2,istart,iend,data_r2,rcode)
                   else
                      print *,'(3a,i10)','Skipping ',trim(varnam),' because it is type=',ivtype2
                      cycle VAR_LOOP
                   end if
                   if (rcode .ne. 0) then
                      print *,'Error reading 2nd id_var= ',id_var,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                endif
             elseif (var_type .eq. ncdouble)  then   !get_real_double
                allocate (data_dp_r(istart(1):iend(1),istart(2):iend(2),istart(3):iend(3), &
                     & istart(4):iend(4)))
                call ncvgt( cdfid,id_var,istart,iend,data_dp_r,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
                data_r = data_dp_r
                if (time_diff .gt. 0) then
                   istart(ndims) = time_diff
                   iend(ndims) = time_diff
                   call ncvgt( cdfid,id_var,istart,iend,data_dp_r,rcode)
                   if (rcode .ne. 0) then
                      print *,'Error reading time_diff= ',time_diff,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                   data_r2 = data_dp_r
                endif
                if (two_files) then
                   if (ivtype2 .eq. ncdouble) then
                      call ncvgt( cdfid2,id_var2,istart,iend,data_dp_r,rcode)
                      data_r2=data_dp_r
                   elseif (ivtype2 .eq. ncfloat) then
                      call ncvgt( cdfid2,id_var2,istart,iend,data_r2,rcode)
                   else
                      print *,'(3a,i10)','Skipping ',trim(varnam),' because it is type=',ivtype2
                      cycle VAR_LOOP
                   end if
                   if (rcode .ne. 0) then
                      print *,'Error reading 2nd id_var= ',id_var,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                endif
                deallocate (data_dp_r)
             elseif (var_type .eq. nclong) then   !get_int
                allocate (data_i(istart(1):iend(1),istart(2):iend(2),istart(3):iend(3), &
                     & istart(4):iend(4)))
                call ncvgt( cdfid,id_var,istart,iend,data_i,rcode)
                if (rcode .ne. 0) then
                   print *,'Error reading 1st id_var= ',id_var,' varnam=',trim(varnam)
                   cycle VAR_LOOP
                endif
                data_r = data_i
                if (time_diff .gt. 0) then
                   istart(ndims) = time_diff
                   iend(ndims) = time_diff
                   call ncvgt( cdfid,id_var,istart,iend,data_i,rcode)
                   if (rcode .ne. 0) then
                      print *,'Error reading time_diff= ',time_diff,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                   data_r2 = data_i
                endif
                if (two_files) then
                   if (ivtype2 .ne. nclong) then
                      print *,'(3a,i10)','Skipping ',trim(varnam),' because it is type=',ivtype2
                      cycle VAR_LOOP
                   end if
                   call ncvgt( cdfid2,id_var2,istart,iend,data_i,rcode)
                   if (rcode .ne. 0) then
                      print *,'Error reading 2nd id_var= ',id_var,' varnam=',trim(varnam)
                      cycle VAR_LOOP
                   endif
                   data_r2 = data_i
                endif
                deallocate (data_i)
             else
                cycle VAR_LOOP !skip non-supported variable types
             endif
             if (time_diff .gt. 0 .or. two_files) data_r = data_r2 - data_r
             sample_value_r = data_r(middel(1),middel(2),middel(3),iend(4))
             write(*,301)varnam,nDims-1,trim(att_sav(order_place)),      &
                  iend(1),iend(2),iend(3),                        &
                  sample_value_r,trim(att_sav(unit_place))
             if (by_level .and. ndim2 .eq. 3) then
                write (*,'(a4,a10,4a18)') 'Lev',' N ',' Min ',' Max ',' Bias ',' Rms '
                do i=1,iend(3)
                   call stats(data_r(:,:,i,iend(4)),len_2d, &
                        & n_stats, min_stats, max_stats, bias_stats, rms_stats)
                   write (*,'(i4,i10,4G18.10E2)') i, &
                        & n_stats, min_stats, max_stats, bias_stats, rms_stats
                end do
             else
                call stats(data_r,len_3d, &
                     & n_stats, min_stats, max_stats, bias_stats, rms_stats)
                write (*,'(a,i10,4G18.10E2)') ' N , Min , Max , Bias , Rms: ', &
                     & n_stats, min_stats, max_stats, bias_stats, rms_stats
             endif
             if (allocated(data_r)) deallocate (data_r)
             if (allocated(data_r2)) deallocate (data_r2)
          end do VAR_LOOP
       enddo TIME_LOOP

301    format(A17,"  ",i2,"  ",A3,"  ",3(1x,i4),"  ",G18.10E2,"  ",A)
!===========================================================================================
    endif                ! end of options

    call ncclos(cdfid,rcode)
    if (two_files) call ncclos(cdfid2,rcode)

    print*,"  "
    print*,"   --- End of input file ---   "

  end subroutine get_info_from_cdf
!------------------------------------------------------------------------------

  subroutine stats(x,n,out_n,out_min,out_max,out_bias,out_rms)
    implicit none
    integer, intent(in) :: n
    real, intent(in) :: x(n)
    integer, intent(out) :: out_n
    real, intent(out) :: out_min,out_max,out_bias,out_rms

    integer :: i

    out_n = n
    out_min=minval(x)
    out_max=maxval(x)
    if (max(out_max,-out_min) .gt. sqrt(HUGE(x))/real(out_n)) then
       out_bias = -999.99
       out_rms  = -999.99
    else
       out_bias=sum(x)/real(out_n)
       out_rms=sqrt(max(0.,sum(x*x)/real(out_n)))
    endif
    return
  end subroutine stats
