wrfstats <- list(
	id="$Id: wrfstats.q,v 1.2 2005/04/04 16:57:15 trn Exp $",
	date=date(),
	comment="Contains functions for processing wrfstats output" )

summarize.wrfstats <- function(stats,global.sums=T)
{
#!# $Id: wrfstats.q,v 1.2 2005/04/04 16:57:15 trn Exp $
#!# Combine wrfstats from multiple time periods
#!# Input: stats - object returned by read.all.wrfstats
#!#        global.sums - T/F: combine working sums, or simply average the averages
out.stats <- array(NA,dim=dim(stats)[1:2],dimnames=list(dimnames(stats)[[1]],dimnames(stats)[[2]]))
out.stats [,'Min'] <- apply(stats[,'Min',,drop=F],MAR=1,FUN=min,na.rm=T)
out.stats [,'Max'] <- apply(stats[,'Max',,drop=F],MAR=1,FUN=max,na.rm=T)
if (global.sums) {
  out.stats [,'Bias'] <- apply(stats[,'Bias',,drop=F]*stats[,'N',,drop=F],MAR=1,FUN=sum,na.rm=T)
  out.stats [,'Rms'] <- apply(stats[,'Rms',,drop=F]^2*stats[,'N',,drop=F],MAR=1,FUN=sum,na.rm=T)
  out.stats [,'N'] <- apply(stats[,'N',,drop=F],MAR=1,FUN=sum,na.rm=T)
  out.stats [,'Bias'] <- out.stats [,'Bias'] / out.stats [,'N']
  out.stats [,'Rms'] <- sqrt(out.stats [,'Rms'] / out.stats [,'N'])
} else {
  out.stats [,'N'] <- apply(stats[,'N',,drop=F],MAR=1,FUN=mean,na.rm=T)
  out.stats [,'Bias'] <- apply(stats[,'Bias',,drop=F],MAR=1,FUN=mean,na.rm=T)
  out.stats [,'Rms'] <- apply(stats[,'Rms',,drop=F],MAR=1,FUN=mean,na.rm=T)
}
out.stats
}

plot.wrfstats <- function(stats,x=dimnames(stats)[[1]][1],itimes=1:dim(stats)[3],
			  plot.stats=dimnames(stats)[[2]][-1],
			  ylab=paste(plot.stats,collapse=', '),xlab='times',main=x,
			  xlim=range(itimes,na.rm=T),ylim=range(stats[x,plot.stats,itimes],na.rm=T),
			  lty=1:length(plot.stats),col=1:length(plot.stats))
{
#!# $Id: wrfstats.q,v 1.2 2005/04/04 16:57:15 trn Exp $
#!# plot up time series of stats as read in by read.all.wrfstats
plot(xlim,ylim,type='n',xlab=xlab,ylab=ylab,main=main)
lines(xlim,c(0,0),lty=1,lwd=2)
if (length(lty) < length(plot.stats)) lty <- rep(lty,length(plot.stats))
if (length(col) < length(plot.stats)) col <- rep(col,length(plot.stats))
for (i in 1:length(plot.stats)) lines(itimes,stats[x,plot.stats[i],itimes],lty=lty[i],col=col[i])
}

read.one.wrfstats <- function(fname,start.line=NULL,end.line=NULL,time.line=NULL,timestr=NULL,nflds=50)
{
#!# $Id: wrfstats.q,v 1.2 2005/04/04 16:57:15 trn Exp $
#!# Reads output from stats_wrf_nc (without -bylevel flag) for one time period
#!# By default, looks for first "TIME:" line in output, and assumes 50 fields

  if (is.null(start.line)) {
    if (is.null(time.line)) time.line <- unix(paste('grep -n "TIME:"',fname))[1]
    start.line <- as.numeric(unix(paste("echo '",time.line,"' | sed -e 's/:.*//' ",sep="")))
    if (is.null(timestr)) timestr <- unix(paste("echo '",time.line,"' | sed -e 's/^[0-9]*: *TIME: *//' ",sep=""))
  }
  if (is.null(end.line)) end.line <- start.line + nflds*2
  start.p1 <- start.line + 1
  nlines <- end.line - start.line
  fields <- as.character(unix(paste('tail +',start.p1,' ',fname,' | head -',
				    nlines,' | grep -v "N , Min , Max" | cut -c1-8',sep='')))
  num.lines <- as.character(unix(paste('tail +',start.p1,' ',fname,' | head -',
				       nlines,' | grep "N , Min , Max"',sep='')))
  stats <- array(NA,dim=c(length(fields),5),dimnames=list(fields,c('N','Min','Max','Bias','Rms')))
  start.col <- 31
  widths <- c(10,rep(18,4))
  for (i in 1:dim(stats)[2]) {
    stats[,i] <- as.numeric(substring(num.lines,start.col,start.col+widths[i]))
    start.col <- start.col+widths[i]
  }
  list(fname=fname,start.line=start.line,end.line=end.line,timestr=timestr,stats=stats)
}

read.all.wrfstats <- function(fname,time.lines=NULL,...)
{
#!# $Id: wrfstats.q,v 1.2 2005/04/04 16:57:15 trn Exp $
#!# Reads output from stats_wrf_nc (without -bylevel flag) for multiple time periods
#!# By default, looks for all "TIME:" lines in output, 
#!# and assumes the default no of fields (see read.one.wrfstats)
  if (any(is.null(time.lines))) time.lines <- unix(paste('grep -n "TIME:"',fname))
  out.stats <- NULL
  times <- character(0)
  for (i in 1:length(time.lines)) {
    time.line <- time.lines[i]
    tmp.stats <- read.one.wrfstats(fname,time.line=time.line,...)
    if (is.null(out.stats)) out.stats <- array(NA,dim=c(dim(tmp.stats$stats),length(time.lines)))
    times <- c(times,tmp.stats$timestr)
    out.stats[,,i] <- tmp.stats$stats[]
  }
  if (length(times) == length(time.lines)) {
    dimnames(out.stats) <- list(dimnames(tmp.stats$stats)[[1]],dimnames(tmp.stats$stats)[[2]],
				times)
  } else {
    dimnames(out.stats) <- list(dimnames(tmp.stats$stats)[[1]],dimnames(tmp.stats$stats)[[2]])
  }
  out.stats
}
