! Use emacs f90-mode:   -*- mode:f90  -*-
PROGRAM interp_terrain

!-------------------------------------------------------------------------------
! Simple program to read in lat,lon,msl of points and output terrain height and agl
!
! $Log: interp_terrain.f,v $
! Revision 1.1  2007/12/03 18:15:04  trn
! Added terrain interpolation program
!
!
! Initial revision: based on chk_data.f, Revision 1.9  2007/04/17 17:40:12  trn
!
!-------------------------------------------------------------------------------

  use interp_subs

  implicit none

  REAL,          ALLOCATABLE :: RDATA(:,:)   
  CHARACTER(1),  ALLOCATABLE :: CPACK(:)

  integer, parameter :: max_times=20
  CHARACTER(4)               :: KVAR, MODEL 
  CHARACTER(50)              :: LABEL, var_names(max_times)
  character(len=50)          :: lat_name='LAT',lon_name='LON',msl_name='MSL',agl_name='AGL'
  character(len=50)          :: ter_name='TER', outfmt
  real                       :: var_vals(max_times)
  integer                    :: lat_ind=0,lon_ind=0,msl_ind=0,agl_ind=0,ter_ind=0, nvars_in, nvars_out
  CHARACTER(256)             :: FILE, intimes, outtimes, inline, outline
  integer, parameter :: maxlenh=6074
  CHARACTER(maxlenh)            :: HEADER
  LOGICAL                    :: FTEST
  
  integer :: i, icurrent, ierr_interp, indblank, indend, indmax, indstart

  INTEGER  :: NEXP,i1,i2,di,j1,j2,dj, kret, k, krec, IY,IM,ID,IH,IF,KL
  integer :: IYR,IMO,IDA,IHR,IFC,nxy,len
  integer ::          ICX,       MN,                                              &
         NX,       NY,        NZ,                                              &
         K_FLAG,   LENH

  integer :: ierr, itime

  real ::  PREC,VAR1,                                                 &
         POLE_LAT, POLE_LON,  REF_LAT,                                         &
         REF_LON,  SIZE,      ORIENT,                                          &
         TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
         SYNC_LAT, SYNC_LON,  DUMMY

  integer :: iroot,ivar,nvars
  integer, allocatable :: outunits(:)
  character (len=4) :: outvar
  character (len=256) :: outfname
  real :: xi_coord, xj_coord
  logical :: use_16, continue_reading, found_outvar
  integer :: kol, l, num_varb
  real :: height

  integer :: iarg, narg, iargc, iprint, iline
  CHARACTER(256)             :: option, fname
  logical :: lhelp
!-------------------------------------------------------------------------------
  INTERFACE
  SUBROUTINE UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1,i1,i2,di,j1,j2,dj, &
       &            use_16)
  CHARACTER(50),INTENT(IN)  :: label
  CHARACTER(1),INTENT(IN)  :: CPACK(:)  
  INTEGER,     INTENT(IN)  :: NX,NY,NEXP,i1,i2,di,j1,j2,dj,icx,mn
  REAL,        INTENT(OUT) :: RDATA(NX,NY)   
  REAL,        INTENT(IN)  :: PREC,VAR1
  logical, intent (in) :: use_16
  END SUBROUTINE
  END INTERFACE
!-------------------------------------------------------------------------------

  i1=1
  di=50
  i2=-1
  j1=1
  dj=50
  j2=-1
  use_16 = .FALSE.

!=>check for command line arguments
  NARG=IARGC()

  lhelp = narg .eq. 0

!     process options
  iarg = 0
  FILE=' '
  intimes=' '
  outtimes=' '
  outvar = 'SHGT'  !'DWRF' for 16-bit files
  iprint=2
  do while (iarg .lt. narg .and. .not. lhelp)
     iarg=iarg+1
     CALL GETARG(iarg,option)
     if (option(1:1) .ne. '-') exit
     iarg=iarg+1
     CALL GETARG(iarg,fname)
     SELECT CASE (option)
     CASE ('-V','-v')
        read(fname,*,iostat=ierr) iprint
        if (ierr .ne. 0) then
           write (*,*) 'Error decoding iprint from -V option:',trim(fname)
           lhelp = .TRUE.
        endif
     CASE ('-A','-a')
        FILE = fname
     CASE ('-I','-i')
        intimes = fname
     CASE ('-O','-o')
        outtimes = fname
     CASE ('-T','-t')
        outvar = fname
     CASE ('-P','-p')
        SELECT CASE (fname)
        CASE ('16')
           use_16 = .TRUE.
        CASE ('8')
           use_16 = .FALSE.
        CASE DEFAULT
           write (*,*) 'Invalid value for -P option:',trim(fname)
           lhelp=.TRUE.
        end SELECT
     CASE DEFAULT
        write (*,*) 'Invalid option:',trim(option)
        lhelp=.TRUE.
     END SELECT
  end do
  lhelp = lhelp .or. FILE .eq. ' ' .or. intimes .eq. ' ' .or. outtimes .eq. ' '
  lhelp = lhelp .or. iarg .gt. narg
  write (*,'(a)') 'Running interp_terrain $Revision: 1.1 $ with the following options:'
  write (*,'(a,t50,a)') 'REQUIRED: Input ARL file [-A]:',trim(FILE)
  write (*,'(a,t50,a)') 'REQUIRED: Input times object file [-I]:',trim(intimes)
  write (*,'(a,t50,a)') 'REQUIRED: Output times object file [-O]:',trim(outtimes)
  write (*,'(a,t50,l1)') 'Flag for 16-bit ARL precision [-P 16/-P 8]: ',use_16
  write (*,'(a,t50,a)') 'Terrain variable name',trim(outvar)
  write (*,'(a,t50,i10)') 'Printout control [-V]:',iprint
  
  if (iprint .lt. 2) i1 = -1

  if (lhelp) then
     write (*,'(a)') 'Usage: '
     write (*,'(2a)') '  interp_terrain -A arl_filename -I in_times_filename', &
          &' -O out_times_filename [-T terrain_name] [-P 16/8] [-V iprint]'
     stop 'Usage'
  endif

! test for meteo file existence
  INQUIRE(FILE=FILE,EXIST=FTEST)
  IF(.NOT.FTEST)THEN
     WRITE(*,*)'Unable to find file: ',trim(FILE)
     STOP
  END IF

! open file to decode the standard label (50) plus the
! fixed portion (108) of the extended header
  OPEN(10,FILE=FILE,RECL=158,ACCESS='DIRECT',FORM='UNFORMATTED')

! decode the standard portion of the index record
  READ(10,REC=1)LABEL,HEADER(1:108)
  READ(LABEL,'(5I2,4X,A4)')IYR,IMO,IDA,IHR,IFC,KVAR
  WRITE(*,'(A,4I5)')'Opened file       : ',IYR,IMO,IDA,IHR

  IF(KVAR.NE.'INDX')THEN
     WRITE(*,*)'WARNING Old format meteo data grid'
     WRITE(*,*)LABEL
     WRITE(*,*)HEADER(1:108)
     STOP
  END IF

! decode extended portion of the header
  READ(HEADER(1:108),'(A4,I3,I2,12F7.0,3I3,I2,I4)',iostat=ierr)                &
         MODEL,    ICX,       MN,                                              &
         POLE_LAT, POLE_LON,  REF_LAT,                                         &
         REF_LON,  SIZE,      ORIENT,                                          &
         TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
         SYNC_LAT, SYNC_LON,  DUMMY,                                           &
         NX,       NY,        NZ,                                              &
         K_FLAG,   LENH

  if (ierr .ne. 0) then
     WRITE(*,'(a)') 'ERROR: decoding first header, cannot recover'
     WRITE(*,'(2a)') 'LABEL=',LABEL
     WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
  end if

! close file and reopen with proper length
  CLOSE (10)
  NXY = NX*NY
  if (use_16) nxy=2*nxy
  LEN = NXY+50
  OPEN(10,FILE=FILE,RECL=LEN,ACCESS='DIRECT',FORM='UNFORMATTED')
  if (i1 .lt. 0) i1=nx+1
  if (j1 .lt. 0) j1=ny+1
  if (i2 .lt. 0) i2=nx
  if (j2 .lt. 0) j2=ny

! print file diagnostic
  if (iprint .gt. 0) WRITE(*,'(A,4I8)')'Grid size and lrec: ',NX,NY,NXY,LEN
  if (iprint .gt. 0) WRITE(*,'(A,I5)') 'Header record size: ',LENH

! allocate array space
  ALLOCATE (RDATA(NX,NY))   
  ALLOCATE (CPACK(NXY))

! read entire file and print headers
  KREC=1
  itime = 0
  continue_reading = .TRUE.
  found_outvar = .FALSE.
  READ_LOOP: do while (continue_reading)
     READ(10,REC=KREC,iostat=ierr)LABEL,(CPACK(K),K=1,NXY)
     if (ierr .ne. 0) then
        write (*,*) 'Read error ierr= ',ierr,' at krec= ',krec
        continue_reading=.FALSE.
        cycle READ_LOOP
     end if
     
     READ(LABEL,'(6I2,2X,A4,I4,2E14.7)',iostat=ierr) IY,IM,ID,IH,IF,KL,  &
                                             KVAR,NEXP,PREC,VAR1

     if (ierr .ne. 0) then
        WRITE(*,'(a)') 'ERROR: decoding LABEL, going on to next record'
        WRITE(*,'(2a)') 'LABEL=',LABEL
        WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
        krec=krec+1
        cycle READ_LOOP
     end if

     if (kvar .eq. 'INDX' .or. itime .eq. 0) then
        if (lenh .gt. maxlenh .or. lenh .gt. nxy) then
           write (*,*) 'Warning: ignoring header characters from ',min(maxlenh,nxy), &
                & ' to ',lenh
           lenh=min(maxlenh,nxy)
        end if
        do k=1,lenh
           header(k:k) = cpack(k)
        enddo
        if (iprint .gt. 0) WRITE(*,'(A,1x,a)') LABEL,header(1:108)
! decode extended portion of the header
        READ(HEADER(1:108),'(A4,I3,I2,12F7.0,3I3,I2,I4)',iostat=ierr)              &
             MODEL,    ICX,       MN,                                              &
             POLE_LAT, POLE_LON,  REF_LAT,                                         &
             REF_LON,  SIZE,      ORIENT,                                          &
             TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
             SYNC_LAT, SYNC_LON,  DUMMY,                                           &
             NX,       NY,        NZ,                                              &
             K_FLAG,   LENH
        if (ierr .ne. 0) then
           WRITE(*,'(a)') 'ERROR: decoding HEADER'
           WRITE(*,'(2a)') 'LABEL=',LABEL
           WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
        end if
        itime = itime + 1
     else
        if (iprint .gt. 0) WRITE(*,'(A)')LABEL
        if (kvar .eq. outvar) then
           CALL UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1, &
                &    i1,i2,di,j1,j2,dj, &
                &    use_16)
           found_outvar = .TRUE.
           continue_reading = .FALSE.
           cycle READ_LOOP
        endif
     end if
     KREC=KREC+1
  end do READ_LOOP
  
  if (.not. found_outvar) then
     write (*,*) 'outvar =',outvar,' not found in input, aborting'
     stop 'outvar not found in input, aborting'
  end if
  
! open input and output times files
  write (*,*) 'Reading input times object from file: ',trim(intimes)
  open(unit=11,file=intimes,status='old',iostat=ierr)
  if (ierr .ne. 0) then
     write (*,*) 'Error opening intimes file: ',trim(intimes),' ierr=',ierr
     Stop 'error opening intimes file'
  end if
  write (*,*) 'Writing output times object to file: ',trim(outtimes)
  open(unit=12,file=outtimes,status='unknown',iostat=ierr)
  if (ierr .ne. 0) then
     write (*,*) 'Error opening outtimes file: ',trim(outtimes),' ierr=',ierr
     Stop 'error opening outtimes file'
  end if
  
! Read and decode header

  read(11,'(a)',iostat=ierr) inline
  if (ierr .ne. 0) then
     write (*,*) 'Error reading header from intimes file: ',trim(intimes),' ierr=',ierr
     Stop 'error reading header from intimes file'
  end if

  indmax = len_trim(inline)
  indblank = 0
  indstart = 0
  nvars_in = 0
  icurrent = 1
  do while ( nvars_in .lt. max_times-2 .and. icurrent .lt. indmax)
!    start of variable name: look for first non-blank character in remaining subtring
     indstart = verify(inline(icurrent:indmax),' ')
     if (indstart .eq. 0) exit
     indstart = indstart + icurrent - 1
!    end of variable name: look for first blank character following the variable name
     indblank = index(inline(indstart:indmax),' ')
     if (indblank .eq. 0) indblank = indmax-indstart+2 !case of the last variable name
     indend = indblank + indstart - 2
!    store variable name, go on to next one
     nvars_in = nvars_in + 1
     var_names(nvars_in) = inline(indstart:indend)
     write (*,*) 'Decoded input variable ',nvars_in,' :',trim(var_names(nvars_in))
     if (trim(var_names(nvars_in)) .eq. trim(lat_name)) lat_ind=nvars_in
     if (trim(var_names(nvars_in)) .eq. trim(lon_name)) lon_ind=nvars_in
     if (trim(var_names(nvars_in)) .eq. trim(msl_name)) msl_ind=nvars_in
     icurrent = indend + 1
  end do

  if (lat_ind .eq. 0 .or. lon_ind .eq. 0 .or. msl_ind .eq. 0) then
     write (*,*) 'Missing a required input variable (index=0):'
     write (*,*) ' latitude:   name=',trim(lat_name),' index=',lat_ind
     write (*,*) ' longitude:  name=',trim(lon_name),' index=',lon_ind
     write (*,*) ' MSL height: name=',trim(msl_name),' index=',msl_ind
     stop 'Missing a required input variable'
  end if
  
! Generate and output header to output file

  ter_ind = nvars_in + 1
  var_names(ter_ind) = ter_name
  agl_ind = nvars_in + 2
  var_names(agl_ind) = agl_name
  nvars_out = nvars_in + 2
  outline = trim(inline) // ' ' // trim(ter_name) // ' ' // trim(agl_name)

  write (12,'(a)') trim(outline)
  write (outfmt,'(a,i2,a)') '(',nvars_out,'f12.4)' 

  ierr = 0
  iline = 1
  do while ( ierr .eq. 0)
     iline = iline + 1
     read(11,*,iostat=ierr) (var_vals(i), i=1,nvars_in)
     if (ierr .ne. 0) then
        write (*,*) 'Hit read error at line ',iline,' of ',trim(intimes),' with ierr=',ierr
     else
        call interp_2d(var_vals(lon_ind),var_vals(lat_ind),rdata,nx,ny,header,maxlenh, &
             & var_vals(ter_ind),ierr_interp,iprint)
        if (ierr_interp .ne. 0) then
           write (*,*) 'Warning: ierr_interp= ',ierr_interp
        else
           var_vals(agl_ind) = var_vals(msl_ind) - var_vals(ter_ind)
           write (12,outfmt) (var_vals(i),i=1,nvars_out)
        end if
     end if
  end do

  write (*,*) 'Normal Program Stop'

end PROGRAM interp_terrain

!-------------------------------------------------------------------------------

SUBROUTINE UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1,i1,i2,di,j1,j2,dj, &
       &            use_16)

  implicit none

  CHARACTER(50),INTENT(IN)  :: label
  CHARACTER(1),INTENT(IN)  :: CPACK(:)  
  INTEGER,     INTENT(IN)  :: NX,NY,NEXP,i1,i2,di,j1,j2,dj,icx,mn
  REAL,        INTENT(OUT) :: RDATA(NX,NY)   
  REAL,        INTENT(IN)  :: PREC,VAR1
  logical, intent (in) :: use_16

  real ::rmin, rmax, rmean

  real :: scale, vold, rmean_i, rmin_i, rmax_i
  integer :: indx, tstj, j, tsti, i, itmp, itmp1, itmp2

! only required when dealing with SUN F90 compiler
! replace ICHAR below with internally defined JCHAR function
! CHARACTER MYCHR*1
! JCHAR(MYCHR)=IAND(ICHAR(MYCHR),255)

  if (.not. use_16) then
     SCALE=2.0**(7-NEXP)
  else
     scale=2.0**(15-NEXP)
  end if
  
  VOLD=VAR1
  INDX=0
  tstj=j1
  rmean = 0.
  rmean_i = 0.
  rmin_i = +1.e10
  rmax_i = -1.e10
  DO J=1,NY
     tsti=i1
     DO I=1,NX
        INDX=INDX+1
! the following 2 lines taken from pakinp.f90:
        if (.not. use_16) then
           ITMP=ICHAR(CPACK(INDX))
           IF(ITMP.LT.0)ITMP=ITMP+256
           RDATA(I,J)=(ITMP-127.)/SCALE+VOLD
        else
           ITMP1=ICHAR(CPACK(INDX))
           INDX=INDX+1
           ITMP2=ICHAR(CPACK(INDX))
           ITMP=ITMP2*256+ITMP1
           IF(ITMP.LT.0)ITMP=ITMP+65536
           RDATA(I,J)=((ITMP-32767.0)/SCALE)+VOLD
        end if
        
        VOLD=RDATA(I,J)
! the following line taken from pakinp.f90:
        IF(ABS(RDATA(I,J)) .LT. PREC) RDATA(I,J)=0.0
        rmean = rmean + rdata(i,j)
        if (i .le. nx-1 .and. j .le. ny-1) then
           rmean_i=rmean_i+rdata(i,j)
           rmin_i=min(rmin_i,rdata(i,j))
           rmax_i=max(rmax_i,rdata(i,j))
        endif
        IF(I.eq.tsti.AND.J.eq.tstj) then
           WRITE(*,'(3I5,E15.6)')J,I,ICHAR(CPACK(INDX)),RDATA(I,J)
           tsti = tsti + di
           if (tsti .gt. i2) tsti=-99
        endif
     END DO
     IF(J.eq.tstj) then
        tstj=tstj+dj
        if (tstj .gt. j2) tstj=-99
     endif
     VOLD=RDATA(1,J)
  END DO
  rmin = minval(rdata)
  rmax = maxval(rdata)
  rmean = rmean/(nx*ny)
  rmean_i = rmean_i/((nx-1.)*(ny-1.))
  WRITE(*,'(a,2i4,a,6g15.6)') label(1:18),icx,mn,': min,max,mean=',rmin,rmax,rmean, &
       & rmin_i,rmax_i,rmean_i
  
  return

END SUBROUTINE unpack

