#!/bin/csh -f

set tasks_def = ( wrf arl )
switch ($#argv)
case 3:
case 2:
case 1:
case 0:
	echo "$0 aborting because supplied $#argv arguments"
	echo "It needs 4 or more:"
	echo "1 - Forecasts init day (yyyymmdd)"
	echo "2 - Forecasts ending day (yyyymmdd) - init day plus one"
	echo "3 - case (example: case04)"
	echo "4 - ndomains"
	echo "5 or more (optional) - tasks, defaults to: $tasks_def"
	exit (1)
default:
	set iday = $argv[1]
	shift
	set eday = $argv[1]
	shift
        set case = $argv[1]
	shift
        set ndomains = $argv[1]
	shift
endsw

if ($#argv < 1) then
   set tasks = ( $tasks_def )
else
   set tasks = ( $* )
endif

if ($?RUNDIR) then
 set rundir=$RUNDIR
else
 set rundir="/beehive/scratch/day14/trn"
endif
if ($?ARCHIVE) then
 set archive=$ARCHIVE
else
 set archive="/project/p1229/wrf/p1229-${case}"
endif

echo "Running $0 with:"
echo " iday=$iday"
echo " eday=$eday"
echo " tasks=$tasks"
echo " rundir=$rundir"
echo " archive=$archive"

set echo on

foreach task ( $tasks )
  if ($task == "wrf") then
    cd $rundir/saved
    if (-e ${archive}/wrfout/${iday}.tar.gz) then
       echo "Not creating ${archive}/wrfout/${iday}.tar.gz, it already exists"
    else
       tar czf ${archive}/wrfout/${iday}.tar.gz ${iday}*
    endif
  endif
  if ($task == "arl") then
    cd $rundir/arl/$iday
    set idyr=`echo $iday | sed -e 's/....$//'`
    set idmon=`echo $iday | sed -e 's/^....//' -e 's/..$//'`
    set idday=`echo $iday | sed -e 's/^......//'`
    set edyr=`echo $eday | sed -e 's/....$//'`
    set edmon=`echo $eday | sed -e 's/^....//' -e 's/..$//'`
    set edday=`echo $eday | sed -e 's/^......//'`
    if (-e ${archive}/arl/${iday}.tar.gz) then
       echo "Not creating ${archive}/arl/${iday}.tar.gz, it already exists"
    else
       tar czf ${archive}/arl/${iday}.tar.gz wrfout_d??_${idyr}-${idmon}-${idday}_*.CFG_MASS.WRF \
               wrfout_d??_${edyr}-${edmon}-${edday}_*.CFG_MASS.WRF \
               wrfout_d??_${idyr}-${idmon}-${idday}_*.out wrfout_d??_${edyr}-${edmon}-${edday}_*.out
    endif
    @ idom=0
    while ($idom < $ndomains)
       @ idom++
       set domain="d$idom"
       if ($idom < 10) set domain="d0$idom"
       if (-e ${archive}/arl/${domain}.${iday}.arl) then
	echo "Not creating ${archive}/arl/${domain}.${iday}.arl, it already exists"
       else
	cat wrfout_${domain}_${idyr}-${idmon}-${idday}_*.DATA_MASS.WRF wrfout_${domain}_${edyr}-${edmon}-${edday}_*.DATA_MASS.WRF> ${archive}/arl/${domain}.${iday}.arl
       endif
    end
  endif
end
