#!/bin/csh -f

# cvs info: $Id: check_for_nans.csh,v 1.1 2007/04/29 14:47:12 trn Exp $
# Script to check for NaNs in WRF netcdf output files
# Required argument(s): pattern(s) to match wrf output directories

if ($?NWPDIR) then
 echo "Using setting supplied by environment variable NWPDIR"
 set nwp_dir=$NWPDIR
else
 set hostlocal=`hostname`
 switch ($hostlocal)
 case humboldt*:
    echo "Using settings for humboldt"
    set nwp_dir="/project/p1229/wrf/humboldt/NWP"
    breaksw
 case beehive*:
    echo "Using settings for beehive"
    set nwp_dir="$HOME/NWP"
    breaksw
  default:
    echo "Using default settings for host=$hostlocal"
    set nwp_dir="$HOME/NWP"
  endsw
endif
echo "nwp_dir=$nwp_dir"
if (-x $nwp_dir/WRF_util/inhouse_util/stats_wrf_nc == 0) then
   echo "$nwp_dir/WRF_util/inhouse_util/stats_wrf_nc is not there or not executable:"
   /bin/ls -l $nwp_dir/WRF_util/inhouse_util/stats_wrf_nc
   exit 1
endif

if ($#argv > 0) then
   set pattern=$*
else
   echo "Missing required argument(s): pattern(s) to match wrf output directories"
   exit 1
endif

echo "pattern=$pattern"

foreach dir ( ${pattern}* )
set d01=`/bin/ls $dir/wrfout_d01* | tail -1`
echo -n "$d01 : "
$nwp_dir/WRF_util/inhouse_util/stats_wrf_nc -bylevel $d01 |& grep -i NaN | wc -l
set d02=`/bin/ls $dir/wrfout_d02* | tail -1`
echo -n "$d02 : "
$nwp_dir/WRF_util/inhouse_util/stats_wrf_nc -bylevel $d02 |& grep -i NaN | wc -l
end
