#!/usr/bin/perl
# $Id: compare_fcst.pl,v 1.1 2006/01/16 15:43:20 trn Exp $

# this script computes statistics of differences between two WRF forecast GRIB outputs
# GRIB converted to native if needed.
# Splus is used to compute difference stats

use strict;
use File::Basename;
use Getopt::Long ;

umask oct("002");
$|=1; #use unbuffered I/O

my %filesuf=('half' => 'half_3d','full' => 'full_3d','u' => 'u_3d','v' => 'v_3d',
	     'sfc' => 'sfc_2d','pcp' => 'pcp_2d', '2m' => '2m_2d', 'uv10m' => 'uv10m_2d', 
	     'soil' => 'soil_3d');
my @exts=('.hdr','.dat');

# Makes some assumptions about directories under HOME
my $home=$ENV{HOME};
my $wrfutil="$home/NWP/WRF_util/inhouse_util";
my $nativeutil="$home/NWP/native_util";
# for nativestats.csh:
$ENV{S_DIR}="$home/MySwork";
my $nativestats="$home/NWP/generic/splus/src/nativestats.csh";

my ($verbose, $debug, $help, $wrf2_native, $wrf2_grib, $wrf1_grib, $wrf1_native, @types, @dates, $ihr, @hrs, $solhost);
$verbose=1;

my %options = (
	       "verbose!"        => \$verbose, 
	       "debug!"        => \$debug, 
	       "help|?!"         => \$help,
	       "wrf1_native=s"  => \$wrf1_native,
	       "wrf1_grib=s"  => \$wrf1_grib,
	       "wrf2_native=s"  => \$wrf2_native,
	       "wrf2_grib=s"  => \$wrf2_grib,
	       "types=s@"  => \@types,
	       "dates=i@"  => \@dates,
	       "ihr=s"  => \$ihr,
	       "solhost=s"  => \$solhost,
	       "hrs=s@"  => \@hrs,
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "debug!"        => "debugging control", 
	       "help|?!"         => "generate help message",
	       "wrf1_native=s"  => "[REQ: unless wrf1_grib is provided] directory containing wrf_native files",
	       "wrf1_grib=s"  => "[REQ: unless wrf1_native is provided] directory containing wrf_grib files",
	       "wrf2_native=s"  => "[REQ: unless wrf2_grib is provided] directory containing wrf_native files",
	       "wrf2_grib=s"  => "[REQ: unless wrf2_native is provided] directory containing wrf_grib files",
	       "types=s@"  => "types to process",
	       "dates=i@"  => "[REQ:]  Starting date (yyyymmdd) of fcst; if it extends into following
days, then also provide yyyymmdd for day+1, day+2, ...",
	       "ihr=s"  => "Starting hour (2-digit)",
	       "hrs=s@"  => "Fcst hours (2-digit)",
	       "solhost=s"  => "host name for machine running Solaris and Splus",
	      );

$help=1 unless GetOptions (%options);
$help=1 unless @dates; #required
$help=1 unless ($wrf1_native || $wrf1_grib); #required
$help=1 unless ($wrf2_native || $wrf2_grib); #required
$wrf1_native="$home/junk/native1" unless $wrf1_native; #optional
$wrf2_native="$home/junk/native2" unless $wrf2_native; #optional
$solhost="burst" unless $solhost;
@hrs=("00","06","12","18","24","30") unless @hrs;
@types = ('half','full','u','v','sfc','pcp','2m','uv10m','soil') unless @types; #default

my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
$verbose=1 if $help;
$verbose=1 if $debug;
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

my $cmd0=dirname($0);

my $cmd;
my $rundir=`pwd`;
chomp $rundir;

foreach my $test_dir ($wrf1_native,$wrf2_native,"$rundir/$dates[0]") {
  if (!chdir($test_dir)) {
    unlink $test_dir if (-e $test_dir);
    mkdir ($test_dir) or die "Cannot mkdir $test_dir\n";
  }
}

foreach my $fhr (@hrs) {
  chdir ($rundir) or die "Cannot chdir to rundir=$rundir\n";
  my $idate=0;
  my $vhr=$ihr+$fhr;
  while ($vhr > 23) {
    $idate=$idate+1;
    $vhr=$vhr-24;
  }
  my $vdate=$dates[$idate];
  while (length($vhr) < 2) {
    $vhr="0".$vhr;
  }
  my $vyr=substr($vdate,0,4);
  my $vmm=substr($vdate,4,2);
  my $vda=substr($vdate,6,2);

  print "fhr= ".$fhr." vdate= ".$vdate." vhr= ".$vhr."\n";

# look for native file(s), generate if needed
  my ($do_wrf1, $do_wrf2);
  my $this_wrf1grib=$wrf1_grib."/wrfout_d01_".$vyr."-".$vmm."-".$vda."_".$vhr.":00:00";
  my $this_wrf1native=$wrf1_native."/d01_".$fhr."hr";
  my $this_wrf2grib=$wrf2_grib."/wrfout_d01_".$vyr."-".$vmm."-".$vda."_".$vhr.":00:00";
  my $this_wrf2native=$wrf2_native."/d01_".$fhr."hr";
  my ($type, $ext);
  for $type (@types) { 
    for $ext (@exts) {
      $do_wrf1=1 if (! -e $this_wrf1native."_".$filesuf{$type}.$ext);
      print "do_wrf1=$do_wrf1 for $this_wrf1native"."_".$filesuf{$type}.$ext."\n" if $debug;
      $do_wrf2=1 if (! -e $this_wrf2native."_".$filesuf{$type}.$ext);
      print "do_wrf2=$do_wrf2 for $this_wrf2native"."_".$filesuf{$type}.$ext."\n" if $debug;
    }
  }

  for (my $iwrf=1;$iwrf<3;$iwrf++) {
    my $this_wrfgrib   = $iwrf == 1 ? $this_wrf1grib   : $this_wrf2grib ;
    my $this_wrfnative = $iwrf == 1 ? $this_wrf1native : $this_wrf2native ;
    next if ($iwrf == 1 && !$do_wrf1);
    next if ($iwrf == 2 && !$do_wrf2);
    my ($this_file, $this_dir, $this_suf) = fileparse($this_wrfnative,'');
    if (!chdir ($this_dir)) {
      warn "Could not chdir $this_dir \n";
    } else {
      $cmd="$nativeutil/wrf2native.pl -gribfile $this_wrfgrib -nativefile $this_file";
      $cmd="$cmd -parmfile parm_wrfgrib_stilt";
      print "At ",scalar(localtime)," running $cmd \n" if $verbose;
      warn "Error running $cmd\n" if system($cmd);
    }
  }

  chdir ($rundir) or die "Cannot chdir to rundir=$rundir\n";
  for my $type (@types) {
    $cmd="rsh -n $solhost "."'("."cd $rundir; $nativestats ".$this_wrf1native."_".$filesuf{$type}." ".$this_wrf2native."_".$filesuf{$type};
    $cmd="$cmd debug" if $debug;
    $cmd=$cmd.")'";
    print "At ",scalar(localtime)," running $cmd \n" if $verbose;
    warn "Error running $cmd\n" if system($cmd);
    my $outstats=basename($this_wrf1native)."_".$filesuf{$type}.".".basename($this_wrf2native)."_".$filesuf{$type}.".stats";
    my $newstats=basename($this_wrf1native)."_".$filesuf{$type};
    $newstats =~ tr/_-/../;
    $cmd="/bin/cp $outstats $dates[0]/$newstats";
    print "At ",scalar(localtime)," running $cmd \n" if $verbose;
    warn "Error running $cmd\n" if system($cmd);
  }
}
1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script computes statistics of differences between two WRF forecast GRIB outputs
 GRIB converted to native if needed.
 Splus is used to compute difference stats
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
