#!/bin/csh -f

switch ($#argv)
case 2:
case 1:
case 0:
	echo "$0 aborting because supplied $#argv arguments"
	echo "It needs 3 or more:"
	echo "1 - case (example: case04)"
	echo "2 - ndomains"
	echo "3 or more - idays (yyyymmdd)"
        echo "Environment variables used for overriding defaults:"
        echo "  TMPTRN - parent directory of arl/yyyymmdd working directories"
        echo "  ARLDIR - destination directory of dnn.yyyymmdd.arl files"
        echo "  ARCHIVE - used to construct default ARLDIR=ARCHIVE/arl"
	exit (1)
default:
        set case = $argv[1]
	shift
        set ndomains = $argv[1]
	shift
endsw

set archive_def="/project/p1229/wrf/p1229-${case}"
set hostlocal=`hostname`
switch ($hostlocal)
case humboldt*:
  echo "Using default settings for humboldt"
  set tmpdir_def=/scratch/white/day14/trn
  breaksw
default:
#case beehive*:
  echo "Using default settings for beehive"
  set tmpdir_def=/beehive/scratch/day14/trn
endsw

if ($?TMPTRN) then
 set tmpdir=$TMPTRN
else
 set tmpdir=$tmpdir_def
endif
if ($?ARLDIR) then
  set arldir=$ARLDIR
 else
  if ($?ARCHIVE) then
   set archive=$ARCHIVE
  else
   set archive=$archive_def
  endif
  set arldir=${archive}/arl
endif

echo "Running $0 with:"
echo " case=$case"
echo " ndomains=$ndomains"
echo " tmpdir=$tmpdir"
echo " arldir=$arldir"

set prevdir=""
foreach iday ( $* )
  @ idom=0
  while ($idom < $ndomains)
    @ idom++
    set domain="d$idom"
    if ($idom < 10) set domain="d0$idom"
    echo ""
    echo "Processing iday=$iday, domain=$domain"
    cd $tmpdir/arl/$iday
    if ("a$prevdir" == "a") then
      echo "Not linking to previous iday forecast"
    else
      set prevfile=`/bin/ls -1 ../$prevdir/wrfout_${domain}_????-??-??_*.DATA_MASS.WRF | tail -1`
      echo "ln -s $prevfile ."
      ln -s $prevfile .
    endif
    if (-e ${arldir}/${domain}.${iday}.arl) then
      echo "Not creating ${arldir}/${domain}.${iday}.arl, it already exists"
    else
      echo "Creating ${arldir}/${domain}.${iday}.arl"
      cat wrfout_${domain}_????-??-??_*.DATA_MASS.WRF > ${arldir}/${domain}.${iday}.arl
    endif
  end
  set prevdir=$iday
end

