#!/usr/bin/perl

$|=1;

use strict;
use Time::Local;
use Getopt::Long ;
Getopt::Long::Configure('ignore_case') ;

# to be supplied via args:
my (@domains,$inittime,$interval,$fcstlen,$verbose,$debug,$tmpdir,$NWPDIR,@templates,@levels,$scale,@skip);

$interval=10800;
$fcstlen=30;
$verbose=1;
$debug=0;
$tmpdir=`pwd`;
chomp $tmpdir;
$NWPDIR="$ENV{HOME}/NWP";
$scale=1.94; #knots

my %options = ('verbose!' => \$verbose,       
	       'debug!' => \$debug,       
	       'interval=i' => \$interval,
	       'inittime=i' => \$inittime,
	       'fcstlen=i' => \$fcstlen,
	       'domains=i@' => \@domains,
	       'levels=i@' => \@levels,
	       'templates=s@' => \@templates,
	       'scale=s' => \$scale,
	       'NWPDIR=s' => \$NWPDIR,
	       'tmpdir=s' => \$tmpdir);
my %helpopt = ('verbose!' => "turn on diagnostic output",
	       'debug!'=> "turn on debugging",
	       'inittime=i' => "REQ: initial time of forecast (yyyymmddhh)",
	       'fcstlen=i' => "length of forecast (hours)",
	       'domains=i@' => "Domain numbers to process",
	       'levels=i@' => "Levels (hPa) to process",
	       'templates=s@' => "Grads templates files (in NWPDIR/p1229/wrfv2)",
	       'scale=s' => "velocity scale for barbs",
	       'interval=i' => "output interval (sec)",
	       'NWPDIR=s' => "parent directory of native cvs working directories",
	       'tmpdir=s' => "Working directory for this run");

#my $help=1 unless GetOptions (%options);
my $help;
GetOptions (%options);
$help = 1 if (!$inittime);
$verbose = 1 if ($debug || $help);
@domains=(1,2) unless @domains;
@skip=(2,10) unless @skip;
while (@skip < @domains) {@skip=(@skip,$skip[@skip-1]);}
my $hrint=$interval/3600;
@templates=('grads_narr_upp.template','grads_narr_2m10m.template','grads_stageiv_pcp.template') unless @templates;
@levels=(850,700,500) unless @levels;

my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}

&help(\%options, \%helpopt) if $help;
chdir $tmpdir or die "Cannot chdir $tmpdir\n";
$ENV{NWPDIR} = "$NWPDIR";

# Loop over all fhrs, domains, templates

my $ifhr=0;
my $fhr=$ifhr*$hrint;
my $skip1=$skip[0];
while ($fhr <= $fcstlen) {
  my $p6hr=$fhr-6;
  my $sfhr= sprintf("%02d", $fhr);
  my $sp6hr= sprintf("%02d", $p6hr);
  for (my $idomain=0;$idomain<@domains;$idomain++ ) {
    my $domain=$domains[$idomain];
    my $skip=$skip[$idomain];
    foreach my $template ( @templates ) {
      my $l_template=$template;
      if(($fhr % 3) > 0) {
	#only do narr when it is available:
	$l_template="grads_nonarr_upp.template" if ($l_template eq 'grads_narr_upp.template');
	$l_template="grads_nonarr_2m10m.template" if ($l_template eq 'grads_narr_2m10m.template');
      }
      my $tmproot="grads_narr.".$$;
      my $tmpin1=$tmproot.".in1";
      my $tmpout=$tmproot.".out";
      my $upper=0;
      my $fhrp6=0;
      open (TMPL,"<${NWPDIR}/p1229/wrfv2/${l_template}") or 
	die ("Cannot open <${NWPDIR}/p1229/wrfv2/${l_template}\n");
      open (TMPIN1,">$tmpin1") or die ("Cannot open >$tmpin1\n");
      foreach my $line (<TMPL>) {
	$upper = 1 if ($line =~ /<lev>/);
	$fhrp6 = 1 if ($line =~ /<p6hr>/);
	$line =~ s/<inittime>/$inittime/g;
        $line =~ s/<fhr>/$sfhr/g;
        $line =~ s/<p6hr>/$sp6hr/g;
	$line =~ s/<domain>/d$domain/g;
	$line =~ s/<scale>/$scale/g;
	$line =~ s/<skip>/$skip1/g;
	$line =~ s/<skipd>/$skip/g;
	print TMPIN1 $line;
      }
      close (TMPL) or warn ("Cannot close <${NWPDIR}/p1229/wrfv2/${l_template}\n");;
      close (TMPIN1) or warn ("Cannot close >$tmpin1\n");;
      my $nlev=@levels;
      $nlev=1 unless $upper;
      $nlev=0 if($fhrp6 && ($p6hr < 0 || ($p6hr % 6) > 0)); #only do 6-hrly precip when it makes sense
      print "fhr=$fhr domain=d$domain template=$l_template upper=$upper fhrp6=$fhrp6 nlev=$nlev\n" if $debug;
      my $tmpin2=$tmproot.".in2";
      open (TMPIN1,"<$tmpin1") or die ("Cannot open <$tmpin1\n");
      my @lines=<TMPIN1>;
      close (TMPIN1) or warn ("Cannot close <$tmpin1\n");;
      for (my $ilev=0;$ilev<$nlev;$ilev++) {
	my $lev=$levels[$ilev];
	open (TMPIN2,">$tmpin2") or die ("Cannot open >$tmpin2\n");
	foreach (@lines) {
	  my $line=$_;
	  $line =~ s/<lev>/$lev/g;
	  print TMPIN2 $line;
	}
	close (TMPIN2) or warn ("Cannot close >$tmpin2\n");;
	my $cmd="grads -l -b <$tmpin2 >$tmpout 2>&1";
	print "At ",scalar localtime()," Running $cmd\n" if $verbose;
	my $loc_debug=system($cmd);
	warn ("Error running $cmd\n") if ($loc_debug);
	if (open(TMPOUT,"<$tmpout")) {
	  my @metas=();
	  foreach (<TMPOUT>) {
	    if (/ga-> enable print /) {
	      s/ga-> enable print //;
	      push @metas, $_;
	    }
	  }
	  close(TMPOUT) or warn("Cannot close >$tmpout\n");
	  foreach my $meta (@metas) {
	    $cmd="gxeps -c -i $meta";
	    print "At ",scalar localtime()," Running $cmd\n" if $verbose;
	    my $gx_debug=system($cmd);
	    warn ("Error running $cmd\n") if ($gx_debug);
	    $loc_debug=$loc_debug || $gx_debug;
	  }
	} else {
	  $loc_debug=1;
	  warn ("Cannot open <$tmpout, not running gxeps\n")
	}
	if ($debug || $loc_debug) {
	  rename $tmpin2,"$tmpin2.$sfhr.d$domain.$l_template.$ilev";
	  rename $tmpout,"$tmpout.$sfhr.d$domain.$l_template.$ilev";
	}
      }
      unlink ($tmpin1,$tmpin2,$tmpout) unless $debug;
    }
  }
  $ifhr++;
  $fhr=$ifhr*$hrint;
}

1;

sub help{
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 This script generates GrADS plots from native files from narr and wrfout files:

 Options (REQ denotes required arguments):
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
