#!/bin/perl

use Getopt::Long ;
use File::Basename;

use strict;

my $cmd0 = $0;
my @call = caller(0);
if (@call > 6) {
  $cmd0 = $call[6] if ($call[3] eq "(eval)");
}
my $pwd0 = dirname($cmd0);
$ENV{CONFIG_DIR}= $ENV{CONFIG_DIR}?$ENV{CONFIG_DIR}:"$pwd0" ;
my $sed_config=$ENV{CONFIG_DIR};
$sed_config =~ s#/#\\/#g;
$ENV{GRADS_DIR}= $ENV{GRADS_DIR}?$ENV{GRADS_DIR}:"$ENV{HOME}/NWP/AERNEWS/grads" ;
my $sed_grads=$ENV{GRADS_DIR};
$sed_grads =~ s#/#\\/#g;

my %staxy=('half' => ' ','full' => ' ','u' => 'X','v' => 'Y',
	     'sfc' => ' ','pcp' => ' ', '2m' => ' ', 'uv10m' => ' ', 'soil' => ' ');
my %staz=('half' => ' ','full' =>'Z','u' => ' ','v' => ' ',
	     'sfc' => ' ','pcp' => ' ', '2m' => ' ', 'uv10m' => ' ', 'soil' => ' ');
my %dim3d2d=('half' => '3d','full' =>'3d','u' => '3d','v' => '3d',
	     'sfc' => '2d','pcp' => '2d', '2m' => '2d', 'uv10m' => '2d', 'soil' => '3d');
my %kpds6=('half' => '119','full' =>'119','u' => '119','v' => '119',
	     'sfc' => '1','pcp' => '1', '2m' => '105', 'uv10m' => '105', 'soil' => '112');
my %kpds7=('half' => '-99','full' =>'-99','u' => '-99','v' => '-99',
	     'sfc' => '-99','pcp' => '-99', '2m' => '2', 'uv10m' => '10', 'soil' => '10');
my $vfile;
my $hr;
my $fl=1;
my $nl=30;
my $il=5;
my $templ="$ENV{CONFIG_DIR}/stilt_grads_template.in";
my $help;
my $debug;
my $verbose=1;
my @types;
my $gxps_cmd="gxeps -c";
my $domain="d01";
my %options = (
	       "verbose!"        => \$verbose, 
	       "debug!"        => \$debug, 
	       "help|?!"         => \$help,
	       "vfile=s"       => \$vfile,
	       "hr=s"       => \$hr,
	       "domain=s"       => \$domain,
	       "template=s"       => \$templ,
	       "gxps_cmd=s"       => \$gxps_cmd,
	       "fl=i"       => \$fl,
	       "nl=i"       => \$nl,
	       "il=i"       => \$il,
	       "types=s@"         => \@types
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "debug!"        => "flag for debugging run", 
	       "help|?!"         => "generate help message",
	       "vfile=s"       => "Required: input wrfgrib2arl parameter file",
	       "hr=s"       => "Required: forecast hr (part of input filename)",
	       "domain=s"       => "domain, as dnn",
	       "template=s"       => "template filename",
	       "gxps_cmd=s"       => "meta -> postscript conversion command",
	       "fl=i"       => "Lower limit of level index for 3d variables",
	       "nl=i"       => "Upper limit of level index for 3d variables",
	       "il=i"       => "Increment of level index for 3d variables",
	       "types=s@"       =>"types of variables to process"
	      );
$help=1 unless GetOptions (%options);
$help=1 unless $vfile; #required
$help=1 unless defined($hr); #required (but allow for hr=0)
$verbose=1 if $debug;
@types = keys(%dim3d2d) unless @types; #default
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

open(VFILE,"<$vfile") or die "Cannot open <$vfile\n";
my @lines;
my $endheader;
foreach my $line (<VFILE>) {
  if (!$endheader) {
    $endheader=1 if (substr($line,0,9) eq 'ENDHEADER');
    next;
  } else {
    push @lines, $line;
  }
}
close(VFILE) or die "Cannot close $vfile\n";
my $total_count=0;
foreach my $type (@types) {
  print "In type loop for type=$type\n" if $verbose;
  next unless $kpds6{$type};
  print "  Processing lines for type=$type\n" if $verbose;
  my $type_count=0;
  foreach my $line (@lines) {
    print "Examining line: $line" if $debug;
# Decode each line from VFILE and compare for match against expected values for type:
    my ($A, $staxy, $staz, $dim3d2d, $kpds5, $kpds6, $kpds7, $cnvrt) = &decode_line($line);
    next unless $staxy eq $staxy{$type} && $staz eq $staz{$type} && 
      ($dim3d2d.'d' eq $dim3d2d{$type} || $type eq 'soil' ) && 
	$kpds6 eq $kpds6{$type} && 
	  $kpds7 eq $kpds7{$type};
# Scan GrADS control file for matching variable code
    my $gradsctl="${domain}_" . $hr . "hr_" . $type . "_" . $dim3d2d{$type} . "_grid.ctl";
    print "Examining Grads control file: $gradsctl\n" if $debug;
    if (!open (GRADSCTL,"<$gradsctl") ) {
      print "  Could not open $gradsctl, skipping processing\n" if $verbose;
      next;
    }
    my @ctllines=(<GRADSCTL>);
    close (GRADSCTL) or warn "Could not close $gradsctl\n";
    next unless grep(/^var$kpds5/,@ctllines);
# Edit grads commands template file and run grads and gxeps
    $type_count=$type_count+1;
    my $local_fl = $type eq 'soil' ? '4' : $fl;
    my $local_nl = $dim3d2d eq '3' ? $nl : '1';
    $local_nl = '4' if ($type eq 'soil');
    print "  Processing decoded line:\n\t".
      join(" ","A=".$A, "staxy=",$staxy, "staz=",$staz, "dim3d2d=",$dim3d2d, 
	   "kpds5=",$kpds5, "kpds6=", $kpds6, "kpds7=", $kpds7, "cnvrt=", $cnvrt).
	"\n" if $verbose;
    my $cmd='sed -e "s/<type>/' . $type . '/g"' . ' -e "s/<hr>/' . $hr . '/g"';
    $cmd = $cmd . ' -e "s/<23d>/' . $dim3d2d{$type} . '/g"';
    $cmd = $cmd . ' -e "s/<A>/' . $A . '/g"' . ' -e "s/<G>/' . $kpds5 . '/g"';
    $cmd = $cmd . ' -e "s/<FL>/' . $local_fl . '/g"';
    $cmd = $cmd . ' -e "s/<NL>/' . $local_nl . '/g"' . ' -e "s/<IL>/' . $il . '/g"';
    $cmd = $cmd . ' -e "s/<cnvrt>/' . $cnvrt . '/g"';
    $cmd = $cmd . ' -e "s/<GRADS_DIR>/' . $sed_grads . '/g"';
    $cmd = $cmd . ' -e "s/<CONFIG_DIR>/' . $sed_config . '/g"';
    $cmd = $cmd . ' -e "s/<domain>/' . $domain . '/g"';
    $cmd = $cmd . " $templ > temp-grads-${type}-${type_count}.in";
    print "  Running: $cmd\n" if $verbose;
    if (system($cmd)) {warn "Error running command: $cmd\n";}
    $cmd="grads -b -p < temp-grads-${type}-${type_count}.in > temp-grads-${type}-${type_count}.out";
    print "  Running: $cmd\n" if $verbose;
    if (system($cmd)) {warn "Error running command: $cmd\n";}
    open(TMP, "<temp-grads-${type}-${type_count}.out") or die "Cannot open <temp-grads-${type}-${type_count}.out\n";
    while (<TMP>) {
      next unless /\.meta$/;  #only operate on lines containing metacode filenames
      chomp;
      my $metafile;
      ($metafile = $_) =~ s/^.* +//;
      my $outext=substr($gxps_cmd,0,5) eq "gxeps" ? "eps" : "ps";
      my $outfile=substr($metafile,0,-4).$outext;
      $cmd=$gxps_cmd;
      $cmd="$cmd -v" if ($verbose && $gxps_cmd eq "gxeps -c");
      $cmd="$cmd -i $metafile -o $outfile";
      print "  Running: $cmd\n" if $verbose;
      if (system($cmd)) {warn "Error running command: $cmd\n";}
    }
    unlink "temp-grads-${type}-${type_count}.in" unless $debug;
    unlink "temp-grads-${type}-${type_count}.out" unless $debug;
  }
  print "Processed $type_count variables for type=$type\n" if $verbose;
  $total_count=$total_count+$type_count;;
}

print "Finished. Processed a total of $total_count variables.  Total in vfile:".scalar(@lines)."\n"
  if $verbose;

1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script generates grads plots of stilt variables from wrf2native output files
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}

sub decode_line {
  my $line=$_[0];
  my ($A, $staxy, $staz, $dim23, $kpds5, $kpds6, $kpds7, $cnvrt);
#                      vname              staxy           staz        23d     kpds5  kpds6   kpds7
  if ($line =~ /\s*['](\w\w\w\w)[']\s+[']([ XY])[']\s+[']([ Z])[']\s+(\d)\s+(\d+)\s+(\d+)\s+([-\d]+)\s+([.\d]+)/ ) {
    $A=$1;
    $staxy=$2;
    $staz=$3;
    $dim23=$4;
    $kpds5=$5;
    while (length($kpds5) < 3) {
      $kpds5="0".$kpds5;
    }
    $kpds6=$6;
    $kpds7=$7;
    $cnvrt=$8;
  }
  return ($A, $staxy, $staz, $dim23, $kpds5, $kpds6, $kpds7, $cnvrt);
}
