#!/usr/bin/perl

$|=1;

use strict;
use Time::Local;
use Getopt::Long ;
Getopt::Long::Configure('ignore_case') ;

# to be supplied via args:
my ($rundir,@domains,$inittime,$verbose,$debug,@fhrs,$tmpdir,$NWPDIR);

$debug=0;
$verbose=1;
$tmpdir=`pwd`;
chomp $tmpdir;
$NWPDIR="$ENV{HOME}/NWP";

my %options = ('verbose!' => \$verbose,       
	       'debug!' => \$debug,       
	       'fhrs=i@' => \@fhrs,
	       'inittime=i' => \$inittime,
	       'domains=i@' => \@domains,
	       'rundir=s' => \$rundir,
	       'NWPDIR=s' => \$NWPDIR,
	       'tmpdir=s' => \$tmpdir);
my %helpopt = ('verbose!' => "turn on diagnostic output",
	       'debug!'=> "turn on debugging",
	       'inittime=i' => "REQ: initial time of forecast (yyyymmddhh)",
	       'domains=i@' => "Domain numbers to process",
	       'fhrs=i@' => "REQ: forecast hours to process",
	       'NWPDIR=s' => "parent directory of AERNEWS, native, and arl-format cvs working directories",
	       'rundir=s' => "REQ: Archive directory for WRF grib files, e.g. /project/p1229/wrf/p1229-case06/wrfout",
	       'tmpdir=s' => "Working directory for this run of plot_oneday");

my $help=1 unless GetOptions (%options);
$help = 1 if (!$inittime) ;
$help = 1 if (!$rundir) ;
$help = 1 if (!@fhrs) ;
$verbose = 1 if ($debug || $help);

my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}

&help(\%options, \%helpopt) if $help;
@domains=(1) unless @domains;
chdir $tmpdir or die "Cannot chdir $tmpdir\n";

mkdir ("native") or warn ("Could not mkdir $tmpdir/native\n");
chdir ("native") or die ("Could chdir $tmpdir/native\n");
mkdir ("$inittime") or warn ("Could not mkdir $tmpdir/native/$inittime\n");
chdir ("$inittime") or die ("Could chdir $tmpdir/native/$inittime\n");
if (-e "$tmpdir/var_sample" == 0) {
   print "/bin/cp ${NWPDIR}/arl-format/data2arl/wrfgrib2arl/var_sample $tmpdir/var_sample\n" if $verbose;
   system("/bin/cp ${NWPDIR}/arl-format/data2arl/wrfgrib2arl/var_sample $tmpdir/var_sample");
}

my ($iyr,$imo,$ida,$ihr,$imin,$isec);
if ($inittime =~ /(\d\d\d\d)(\d\d)(\d\d)(\d\d)/ ) {
  $iyr=$1;
  $imo=$2;
  $ida=$3;
  $ihr=$4;
  $imin=0;
  $isec=0;
} else {
  stop("Invalid inittime=$inittime\n");
}
my $initepoch=timegm($isec,$imin,$ihr,$ida,$imo-1,$iyr-1900);
print "inittime=$inittime (Converted to epoch and back:".scalar(gmtime($initepoch)).")\n" if $verbose;

foreach my $hour ( @fhrs ) {
 my $vepoch=$initepoch + 3600*$hour;
 my ($vsec,$vmin,$vhour,$vmday,$vmon,$vyear,$vwday,$vyday) = gmtime($vepoch);
 my $vsyear=1900+$vyear;
 my $vsmon=sprintf("%02d",$vmon+1);
 my $vsday=sprintf("%02d",$vmday);
 my $vshour=sprintf("%02d",$vhour);
 foreach my $domain ( @domains ) {
   my $sdomain="d".sprintf("%02d",$domain);
   my $cmd="${NWPDIR}/native_util/wrf2native.pl -nodorotate";
   $cmd="$cmd -gribfile $rundir/$inittime/wrfout_${sdomain}_${vsyear}-${vsmon}-${vsday}_${vshour}:00:00";
   $cmd="$cmd -nativefile ${sdomain}_${hour}hr -parmfile parm_wrfgrib_stilt";
   print "At ",scalar localtime()," Running $cmd\n" if $verbose;
   warn ("Error running $cmd\n") if (system($cmd));
   $cmd="${NWPDIR}/p1229/wrfv2/plot-stilt.pl -vfile ${tmpdir}/var_sample  -hr ${hour} -domain ${sdomain}";
   print "At ",scalar localtime()," Running $cmd\n" if $verbose;
   warn ("Error running $cmd\n") if (system($cmd));
 }
}

1;

sub help{
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 This script produces grads plots from wrfout files, using:
 - wrf2native.pl to reformat the WRF GRIB files to native
 - plot-stilt.pl to produce the plots, which uses the wrfgrib2arl 
   control file var_sample to select variables to plot

 Options (REQ denotes required arguments):
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
