#!/bin/csh -f

switch ($#argv)
case 1:
case 0:
	echo "$0 aborting because supplied $#argv arguments"
	echo "It needs 2 or more:"
	echo "1 - ndomains"
	echo "2 or more - idays (yyyymmdd)"
	exit (1)
default:
        set ndomains = $argv[1]
	shift
endsw

if ($?CYCLE) then
 set cycle=$CYCLE
else
 set cycle=3
endif
if ($?SAVED) then
 set rundir=$SAVED
else
 set rundir=/beehive/scratch/day14/trn/saved
endif
if ($?TMPTRN) then
 set tmpdir=$TMPTRN
else
 set tmpdir=/beehive/scratch/day14/trn
endif
if ($?NWPDIR) then
 set NWPDIR=$NWPDIR
else
 set NWPDIR=$HOME/NWP
endif
if ($?WRF_NETCDF) then
 set wrf_convert=wrfnc2arl
else
 set wrf_convert=wrfgrib2arl
endif

mkdir $tmpdir/arl
cd $tmpdir/arl
if (-e $tmpdir/var_sample == 0 || -z $tmpdir/var_sample) then
   echo "/bin/cp ${NWPDIR}/arl-format/data2arl/${wrf_convert}/var_sample $tmpdir/var_sample"
   /bin/cp ${NWPDIR}/arl-format/data2arl/${wrf_convert}/var_sample $tmpdir/var_sample
endif

@ kount=0
set idathrs = ( $* )
foreach idathr ( $idathrs )
  set iday=`echo $idathr | sed -e 's/..$//'`
  echo "Processing $idathr in $tmpdir/arl/$iday"
  mkdir $tmpdir/arl/$iday
  cd $tmpdir/arl/$iday
  if ("$wrf_convert" == "wrfgrib2arl") /bin/rm full_levels.txt half_levels.txt
  @ idom=0
  while ($idom < $ndomains)
   @ idom++
   set domain="d$idom"
   if ($idom < 10) set domain="d0$idom"
   @ daykount = 0
   foreach gfile ( $rundir/$idathr/wrfout_${domain}_*:00:00 )
    if ("$wrf_convert" == "wrfgrib2arl") then
     if (-e full_levels.txt && -e half_levels.txt) then
       echo "Reusing existing level files in $tmpdir/arl/$iday"
     else
       if (-e ../full_levels.txt && -e ../half_levels.txt) then
         echo "Linking existing level files from $tmpdir/arl"
	 if (-e full_levels.txt == 0) ln -s ../full_levels.txt .
	 if (-e half_levels.txt == 0) ln -s ../half_levels.txt .
       else
	 ${NWPDIR}/arl-format/data2arl/${wrf_convert}/make_levelfile.pl -grib $gfile -wcode 40
       endif
     endif
    endif
    @ kount++
    if ($daykount > $cycle ) then
     if ("$wrf_convert" == "wrfgrib2arl") then
       ${NWPDIR}/arl-format/data2arl/${wrf_convert}/${wrf_convert} \
       -P $tmpdir/var_sample \
       -H half_levels.txt -F full_levels.txt $gfile >&! $gfile:t.out
     else
       ${NWPDIR}/arl-format/data2arl/${wrf_convert}/${wrf_convert} \
       -P $tmpdir/var_sample $gfile >&! $gfile:t.out
     endif
     foreach dfile ( DATA_*.WRF CFG_*.WRF )
       /bin/mv $dfile $gfile:t.$dfile
     end
    endif
    @ daykount++
   end
  end
end

