#!/usr/bin/perl
# $Id: run_oneday.pl,v 1.22 2007/04/26 13:25:10 trn Exp $

# Top-level script for running a (one-day or shorter) forecast with WRF
# TBD: All control input is via the xml format configuration file
#      The path name of the configuration file is the only argument to this script

use strict;
use File::Basename;
use Getopt::Long ;

umask oct("002");
$|=1; #use unbuffered I/O

my ($verbose, $debug, $help, $dogrib, $dowrf, $itime, $etime, $dt, $fcstlen, $nproc, $config, $rundir, 
    $siwrf, $wrfwrf, $oldrun, $qsub, $peqsub, $sge, $gribtype, $gribdir, $ndomains, $wget_host, $mv_host);
$ndomains=2;
$verbose=1;
$debug=0;
$dogrib=1;
$dowrf=1;
$dt=3;
$fcstlen=24;
$nproc=4;
$config="/project/p1229/wrf/p1229-case04/beehive-wrfsi-domains";
$rundir="/cluster/scratch/day14/trn";
$siwrf="/squall/dap02apr180gb/p1229-carbon/wrf/WRFV2";
$wrfwrf="/squall/dap02apr180gb/p1229-carbon/wrf/WRFV2";
$oldrun="/project/p1229/wrf/p1229-case04/beehive-wrfsi-domains/p1229-case04";
$qsub="qsub -pe mpich 1 /home/trn/bin/qsub_generic.csh";
$wget_host="beehive";
$sge="SGE";

my %options = (
	       "verbose!"        => \$verbose, 
	       "debug!"        => \$debug, 
	       "dogrib!"        => \$dogrib, 
	       "dowrf!"        => \$dowrf, 
	       "help|?!"         => \$help,
	       "itime=i"       => \$itime,
	       "etime=i"       => \$etime,
	       "ndomains=i"       => \$ndomains,
	       "dt=i"       => \$dt,
	       "fcstlen=i"       => \$fcstlen,
	       "nproc=i"       => \$nproc,
	       "config=s"  => \$config,
	       "rundir=s"  => \$rundir,
	       "siwrf=s"  => \$siwrf,
	       "wrfwrf=s"  => \$wrfwrf,
	       "oldrun=s"  => \$oldrun,
	       "qsub=s"  => \$qsub,
	       "peqsub=s"  => \$peqsub,
	       "sge=s"  => \$sge,
	       "wget_host=s"  => \$wget_host,
	       "mv_host=s"  => \$mv_host,
	       "gribtype=s"  => \$gribtype,
	       "gribdir=s"  => \$gribdir
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "debug!"        => "debug run?", 
	       "dogrib!"        => "Flag for doing grib_prep",
	       "dowrf!"        => "Flag for doing wrfprep and run_wrf",
	       "help|?!"         => "generate help message",
	       "itime=i"       => "[REQ:] Starting time (yyyymmddhh)",
	       "etime=i"       => "Ending time (yyyymmddhh)",
	       "ndomains=i"       => "Number of domains",
	       "dt=i"       => "Input data time interval (hours)",
	       "fcstlen=i"       => "Forecast length (hours)",
	       "nproc=i"       => "Number of processors",
	       "config=s"  => "Directory holding configuration files",
               "rundir=s"       => "Parent directory of extdata/ and date/ (for holding WRF output)",
	       "siwrf=s"  => "WRFV2 directory containing the wrfsi directory",
	       "wrfwrf=s"  => "WRFV2 directory containing the WRF code",
	       "oldrun=s"  => "Parent directory of static directory from previous run",
	       "qsub=s"  => "qsub command",
	       "peqsub=s"  => "qsub command for parallel jobs",
	       "sge=s"  => "batch system type",
	       "wget_host=s"  => "Hostname for machine on which to run wget",
	       "mv_host=s"  => "Hostname for machine on which to run the /bin/mv commands for output",
	       "gribtype=s"  => "grib input file type",
	       "gribdir=s"  => "grib input file directory",
	      );
$help=1 unless GetOptions (%options);
$help=1 unless $itime; #required
$etime=$itime unless $etime;
my $yr=substr($itime,0,4);
my $mon=substr($itime,4,2);
my $day=substr($itime,6,2);
my $hr=substr($itime,8,2);
my $eyr=substr($etime,0,4);
my $emon=substr($etime,4,2);
my $eday=substr($etime,6,2);
my $ehr=substr($etime,8,2);
my $fullday=$yr.$mon.$day;
my $fulldayhr=$itime;
$peqsub="qsub -pe mpich $nproc /home/trn/bin/qsub_generic.csh" unless $peqsub;
$gribtype="NARR" unless $gribtype;
$gribdir="/data/ncep-data/narr" unless $gribdir;

my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
$verbose=1 if $help;
$verbose=1 if $debug;
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

my ($cmd, $outline);

my $saved = $ENV{SAVED} ? $ENV{SAVED} : "$rundir/saved";
my $mvcmd="/bin/mv";
my $mkdircmd="mkdir";
my $rmcmd="/bin/rm -fr";
if ($mv_host) {
  $mvcmd="ssh -n $mv_host $mvcmd";
  $mkdircmd="ssh -n $mv_host $mkdircmd";
  $rmcmd="ssh -n $mv_host $rmcmd";
}

if ($dogrib) {

#use or generate rundir, and extdata directory tree:
  foreach my $test_dir ( "$rundir", "$rundir/gribtemp", "$rundir/extdata","$rundir/extdata/work",
			 "$rundir/extdata/log","$rundir/extdata/extprd","$rundir/extdata/static" ) {
    if (! chdir($test_dir)) {
      print "Making new directory $test_dir, old one did not exist\n" if $verbose;
      unlink($test_dir) if (-e $test_dir);
      mkdir ($test_dir, 0777) or warn ("Could not mkdir test_dir=$test_dir\n");
      chdir ($test_dir) or &mydie ("Could not chdir to test_dir=$test_dir\n");
      if ($test_dir eq "$rundir/extdata/static") {
        $cmd="/bin/cp $siwrf/wrfsi/extdata/static/".'*'." $rundir/extdata/static/";
        print "At ",scalar localtime()," Running $cmd\n" if $verbose;
        warn ("Error running $cmd\n") if system($cmd);
        $cmd="/bin/cp $config/grib_prep.nl $rundir/extdata/static";
        print "At ",scalar localtime()," Running $cmd\n" if $verbose;
        warn ("Error running $cmd\n") if system($cmd);
      }
    }
  }

#use or generate saved directory tree (need to use mv_host, allow for latency):
  foreach my $test_dir ( "$saved", "$saved/$fulldayhr" ) {
    if (! chdir($test_dir)) {
      print "Making new directory $test_dir, old one did not exist\n" if $verbose;
      system("$rmcmd $test_dir") if (-e $test_dir);
      warn ("Could not $mkdircmd $test_dir\n") unless system("$mkdircmd $test_dir");
      my $status=0;
      my $count=0;
    CHECKLOOP: while(!$status && $count<30) { #keep checking for up to 5 minutes before giving up
	$status=chdir ($test_dir);
	last CHECKLOOP if ($status);
	print "count=$count : test_dir=$test_dir\n" if $verbose;
	sleep 10;
	$count++;
      }
      &mydie ("Could not chdir to test_dir=$test_dir\n") unless ($status);
    }
  }

  chdir("$rundir/gribtemp") or &mydie("Cannot chdir to $rundir/gribtemp\n");
  opendir(GRIBDIR, $gribdir) or &mydie("gribdir directory cannot be opened: $gribdir \n");
  my @allfiles = sort readdir(GRIBDIR);
  closedir(GRIBDIR);
  my @gribfiles;
  my ($gribfilter,$ifilter,$efilter);
  my $ngribfiles=0;
  if ($gribtype eq "AVN" || $gribtype eq "AVNSFC") {
    #AVN files assumed to reside in gribdir, just link to the ones that are needed
    $gribfilter="fnl_";
    $ifilter=$gribfilter.substr($yr,2,2).$mon.$day;
    $efilter=$gribfilter.substr($eyr,2,2).$emon.$eday;
    foreach (@allfiles) {
      if (/$ifilter/ || /$efilter/) {
	symlink "$gribdir/$_", "$_" or warn ("Could not symlink $gribdir/$_\n");
	@gribfiles=(@gribfiles,$_);
	$ngribfiles++;
      }
    }
  }
  if ($gribtype eq "NARR") {
    # NARR files may reside in gribdir or need to be acquired on the fly
    $gribfilter="narr-a_221_";
    $ifilter=$gribfilter.$yr.$mon.$day.'.*\.grb$';
    $efilter=$gribfilter.$eyr.$emon.$eday.'.*\.grb$';
    my @narr_hrs=( "00","03","06","09","12","15","18","21" );
    # Loop over days for which files are needed:
    for (my $igribday=0;$igribday<2;$igribday++) {
      my $tmpfilter = ( $ifilter, $efilter )[$igribday];
      my $tmpday = ( $yr.$mon.$day, $eyr.$emon.$eday )[$igribday];
      my $ndayfiles=0;
      my @dayfiles;
      foreach (@allfiles) {
	if ( /$tmpfilter/ ) {
	  if (-s "$gribdir/$_") {
	    @dayfiles=(@dayfiles,$_);
	    $ndayfiles++;}
	}
      }
      if ($ndayfiles == @narr_hrs) {
	# have a complete set sitting there, link to those:
	foreach (@dayfiles) {
	  if (!-e $_) {
	    symlink "$gribdir/$_", "$_" or warn ("Could not symlink $gribdir/$_\n");
	    @gribfiles=(@gribfiles,$_);
	  }
	  $ngribfiles++;
	} 
      } else {
	# otherwise, get them on the fly: (NOTE: they will NOT be deleted once the job is done)
	$cmd="ssh -n $wget_host 'cd $rundir/gribtemp;$gribdir/wget-narr.pl $tmpday' ";
# 	print "At ",scalar localtime()," Running $cmd\n" if $verbose;
# 	warn ("Error running $cmd\n") if system($cmd);
	print "At ",scalar localtime()," NOT Running $cmd\n" if $verbose;
	foreach my $tmphr ( @narr_hrs ) {
#	  @gribfiles=(@gribfiles,$gribfilter.$tmpday."_00".$tmphr."_000.grb");
# Do not add these filenames to the list of files to be deleted after completion of grib_prep
	  $ngribfiles++;
	}
      }
    }
  }
  if ($ngribfiles == 0) {
    &mydie("No grib files found for $gribtype in $gribdir with filter=$gribfilter\n");
  }
  # run grib_prep
  chdir("$rundir/extdata") or &mydie("Cannot chdir to $rundir/extdata\n");
  $cmd="$qsub $siwrf/wrfsi/etc/grib_prep.pl -d $rundir/extdata -l $fcstlen -t $dt -s $fulldayhr -f $gribfilter $gribtype";
  print "At ",scalar localtime()," Running $cmd\n" if $verbose;
  $outline=`$cmd`;
  &wait_for_qsub($outline,$sge);
  if (!$debug) {
    chdir("$rundir/gribtemp") or &mydie("Cannot chdir to $rundir/gribtemp\n");
    foreach my $tmpfile (@gribfiles) {
      unlink $tmpfile or warn ("Could not unlink $tmpfile\n");
    }
  }
}

if ($dowrf) {

#generate wrfrun directory tree
  my $wrftop="$rundir/$fulldayhr";
  if (-e $wrftop) {
    $cmd="/bin/rm -fr $wrftop";
    print "At ",scalar localtime()," Running $cmd\n" if $verbose;
    &mydie("Error running $cmd\n") if system($cmd);
  }
  mkdir $wrftop or &mydie ("Could not mkdir $wrftop\n");
  chdir ($wrftop) or &mydie ("Could not chdir $wrftop\n");
  foreach my $test_dir ( "log","siprd","static","wrfprd" ) {
    mkdir "$wrftop/$test_dir" or &mydie("Could not mkdir $wrftop/$test_dir\n");
    if ($test_dir eq "static") {
      my @static_files = ( "README","wrf.nl.README");
      for (my $idomain=1;$idomain <= $ndomains;$idomain++) {
	@static_files=(@static_files,sprintf("static.wrfsi.d%02d",$idomain),
		       sprintf("wrfstatic_d%02d",$idomain));
      }
      foreach my $file ( @static_files ) {
        &mydie ("Error copying $file\n") if system("/bin/cp $oldrun/static/$file $wrftop/static/$file");
      }
      foreach my $file ( "mpi_machines.conf","wrfsi.nl","wrf.nl" ) {
        &mydie ("Error copying $file\n") if system("/bin/cp $config/$file $wrftop/static/$file");
      }
    }
  }

  $cmd="$peqsub $siwrf/wrfsi/etc/wrfprep.pl -d $wrftop -f $fcstlen -t $dt -r p -p $nproc -s $fulldayhr";
  print "At ",scalar localtime()," Running $cmd\n" if $verbose;
  $outline=`$cmd`;
  &wait_for_qsub($outline,$sge);

  # check for vinterp problems
  if ($verbose) {
    $cmd="find $wrftop/log -name \\*.vinterp -exec grep WARNING '{}' \\; -print";
    print "At ",scalar localtime()," Running $cmd\n";
    warn ("Error running $cmd\n") if system($cmd);
  }

  my $maxloop=1;
  RUNLOOP: for (my $runloop=1;$runloop<=$maxloop;$runloop++) {

    $cmd="$peqsub $siwrf/wrfsi/etc/run_wrf.pl -i $wrfwrf/wrfsi -d $wrftop -p $nproc";
    print "At ",scalar localtime()," Running $cmd\n" if $verbose;
    $outline=`$cmd`;
    &wait_for_qsub($outline,$sge);

    # redo if run failed because of socket errors on humboldt:
    my $rerun=1;
    my $rslfile="$wrftop/wrfprd/rsl.out.0000";
    if (open(RSLOUT,"< $rslfile")) {
      while (<RSLOUT>) {
	$rerun=0 if /wrf: SUCCESS COMPLETE WRF/;
      }
      close(RSLOUT) or warn("Could not close RSLOUT: <$rslfile \n");
    }
    print "Apparently unsuccessful run_wrf.pl for runloop=$runloop\n" 
      if ($verbose && $rerun);
    last RUNLOOP unless ($rerun && $runloop<$maxloop);
    sleep 300;
  }

# check for cfl problems
  if ($verbose) {
    $cmd="find $wrftop/wrfprd -name rsl.\\* -exec grep -i cfl '{}' \\; -print";
    print "At ",scalar localtime()," Running $cmd\n";
    warn ("Error running $cmd\n") if system($cmd);
  }

  my $print_part2;
  print "At ",scalar localtime()," moving output files to $saved/$fulldayhr\n" if $verbose;
  warn "Could not mkdir $saved/$fulldayhr\n" unless system("$mkdircmd $saved/$fulldayhr");
  opendir (WRFTOP,"$wrftop");
  foreach (readdir WRFTOP) {
    if ( /^qsub_generic.csh.[oe]/ || /^STDIN/ || /^qsub_hum/ ) {
      if (! system("$mvcmd $wrftop/$_ $saved/$fulldayhr/$_") )
	{$print_part2="successful\n";} else {$print_part2="failed\n"}
      print "$_ : $print_part2" if $verbose;
    }
  }
  closedir (WRFTOP);
  opendir (WRFLOG,"$wrftop/log");
  foreach (readdir WRFLOG) {
    if ( /^[^.]/ ) {
      if (! system("$mvcmd $wrftop/log/$_ $saved/$fulldayhr/$_") )
        {$print_part2="successful\n";} else {$print_part2="failed\n"}
      print "$_ : $print_part2" if $verbose;
    }
  }
  closedir (WRFLOG);
  opendir (WRFPRD,"$wrftop/wrfprd");
  foreach (readdir WRFPRD) {
    if (/^rsl/ || /^wrfout/) {
      if (! system("$mvcmd $wrftop/wrfprd/$_ $saved/$fulldayhr/$_") )
	{$print_part2="successful\n";} else {$print_part2="failed\n"}
      print "$_ : $print_part2" if $verbose;
    }
  }
  closedir (WRFPRD);
}

1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script runs grib_prep, wrfprep, and run_wrf for a WRF forecast
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}

sub mydie {
# Abort script after error exit processing
    my $msg = shift @_;
    print "
=========================================================================
Terminating because of error condition:
$msg
" ;
# Insert exit processing here
    print "
=========================================================================
" ;
    die "Stopped ";
}

sub wait_for_qsub {
    my $outline = shift @_;
    my $qsub_type = shift @_;
    print $outline;
    if ($qsub_type eq "SGE") {
# Sun Grid Engine (SGE)
	my $jobid;
	if ($outline =~ /^(your +job +)(\d+)/i) {
	    $jobid=$2;
	} else {
	    &mydie ("wait_for_qsub: cannot find jobid in outline=\n $outline\n");
	}
	my $finished=0;
	while (1) {
	    my $cmd = "qstat -j ".$jobid;
	    my $outstat=`$cmd`;
	    chomp $outstat;
	    last if ($outstat eq "");
	    sleep 60;
	}
	print "Finished checking for $qsub_type job completion...\n";
	return;
    }
    if ($qsub_type eq "PBS") {
# PBS
	my $jobid;
	if ($outline =~ /^(\d+.\w+)/) {
	    $jobid=$outline;
            chomp $jobid;
	} else {
	    &mydie ("wait_for_qsub: cannot find jobid in outline=\n $outline\n");
	}
	my $finished=0;
	while (1) {
	    my $cmd = "qstat ".$jobid;
	    my $outstat=`$cmd`;
	    chomp $outstat;
	    last if ($outstat =~ /Unknown Job/i || $outstat eq "");
	    sleep 60;
	}
	print "Finished checking for $qsub_type job completion...\n";
	return;
    }
#Fall-through: qsub_type does not match any known batch systems:
    print "
wait_for_qsub: Unknown qsub_type: $qsub_type
               Treating it as a non-batch system (Nothing done)
";
    return;
}
