#!/bin/csh -f

# cvs id: $Id: runnarravn.csh,v 1.14 2007/04/25 12:38:29 trn Exp $

set hrs_def = ( 00 03 06 09 12 15 18 21 )
unset have_hrs
switch ($#argv)
case 4:
case 3:
case 2:
case 1:
case 0:
	echo "$0 aborting because supplied $#argv arguments"
	echo "It needs 5 or more:"
	echo "1 - Forecasts init day (yyyymmdd)"
	echo "2 - Forecasts ending day (yyyymmdd)"
	echo "3 - case (example: case04)"
	echo "4 - ndomains"
	echo "5 - nproc"
	echo "6 or more (optional) - hours, defaults to: $hrs_def"
        echo "Environment variables:"
        echo " RUNDIR = parent directory of saved,gribtemp,extdata,and yyyymmddhh"
        echo " NWPDIR = cvs working directory for p1229 codes"
        echo " CYCLE = how much overlap between adjacent forecasts"
        echo " FCSTLEN = forecast length (hours)"
        echo " WRF_NETCDF = if set, using netcdf format wrfout files"
        echo " ARCHIVE = archive directory (skipped if 'none')"
        echo " TASKS = archive tasks (skipped if 'none', defaults to 'wrf arl')"
	exit (1)
default:
	set iday = $argv[1]
	shift
	set eday = $argv[1]
	shift
        set case = $argv[1]
	shift
        set ndomains = $argv[1]
	shift
        set nproc = $argv[1]
	shift
endsw
if ($#argv < 1) then
   set hrs = ( $hrs_def )
else
   set hrs = ( $* )
endif

set echo on

# set things up for all forecasts from 00z through 21Z; for AVN data (6-hourly), need to go
# to 06Z of the next day
set itime=${iday}00
set etime=${eday}06

set hostlocal=`hostname`
switch ($hostlocal)
case humboldt*:
  echo "Using settings for humboldt"
  set sge="PBS"
  set qsub="qsub_hum -lwalltime=7:00:00"
  set npernode=2
  @ nnode = $nproc
  @ nnode /= $npernode
  set peqsub="$qsub -l nodes=${nnode}:ppn=${npernode}"
#  set peqsub="$qsub -lncpus=$nproc"
  set config="/project/p1229/wrf/p1229-${case}/humboldt-wrfsi-domains"
  set oldrun="$config/p1229-${case}"
  set siwrf="/project/p1229/wrf/humboldt/NWP/WRFV2"
  set wrfwrf="$siwrf"
  set nwp_dir="/project/p1229/wrf/humboldt/NWP"
  set run_dir="/scratch/white/day14/trn"
  set mv_host="anvil"
  setenv MPIRUN `which mpiexec`
  breaksw
case beehive*:
  echo "Using settings for beehive"
  set sge="SGE"
  set qsub="qsub -pe mpich 1 $HOME/bin/qsub_generic.csh"
  set peqsub="qsub -pe mpich $nproc $HOME/bin/qsub_generic.csh"
  set config="/project/p1229/wrf/p1229-${case}/beehive-wrfsi-domains"
  set oldrun="$config/p1229-${case}"
  set siwrf="/squall/dap02apr180gb/p1229-carbon/wrf/WRFV2"
  set wrfwrf="$siwrf"
  set nwp_dir="$HOME/NWP"
  set run_dir="/cluster/scratch/day14/trn"
  set mv_host="anvil"
  breaksw
default:
  echo "No match found for host=$hostlocal"
  exit 1
endsw


if ($?DEBUG) then
 set debug=$DEBUG
else
 set debug=nodebug
endif
if ($?RUNDIR) then
 set rundir=$RUNDIR
else
 set rundir=$run_dir
endif
if ($?NWPDIR) then
 setenv NWPDIR $NWPDIR
else
 setenv NWPDIR $nwp_dir
endif

cd $rundir
/bin/rm -fr extdata
$NWPDIR/p1229/wrfv2/run_oneday.pl -rundir $rundir -dogrib -nodowrf -mv_host $mv_host -itime $itime -etime $etime\
 -fcstlen 30 -dt 3 -gribtype NARR -gribdir "/data/ncep-data/narr" -ndomains $ndomains\
 -qsub "$qsub" -peqsub "$peqsub" -sge $sge -nproc $nproc\
 -config=$config -oldrun=$oldrun -siwrf=$siwrf -wrfwrf=$wrfwrf -$debug
@ kount=0
while ($kount < 2)
   @ kount++
   echo -n "narr extdata/extprd:";date;ls -l extdata/extprd
   if ($kount < 2) sleep 60
end
/bin/rm -fr extdata-narr
/bin/mv extdata extdata-narr
echo -n "extdata-narr/extprd:";date;ls -l extdata-narr/extprd
$NWPDIR/p1229/wrfv2/run_oneday.pl -rundir $rundir -dogrib -nodowrf -mv_host $mv_host -itime $itime -etime $etime\
 -fcstlen 30 -dt 3 -gribtype AVNSFC -gribdir "/data/ncep-data/gfs-anl" -ndomains $ndomains\
 -qsub "$qsub" -peqsub "$peqsub" -sge $sge -nproc $nproc\
 -config=$config -oldrun=$oldrun -siwrf=$siwrf -wrfwrf=$wrfwrf -$debug
@ kount=0
while ($kount < 2)
   @ kount++
   echo -n "avnsfc extdata/extprd:";date;ls -l extdata/extprd
   if ($kount < 2) sleep 60
end
/bin/rm -fr extdata-avnsfc
/bin/mv extdata extdata-avnsfc
echo -n "extdata-avnsfc/extprd:";date;ls -l extdata-avnsfc/extprd
ln -s extdata-narr extdata
echo -n "narr extdata/extprd:";date;ls -l extdata/extprd

if ($?SAVED) then
else
   setenv SAVED "/project/p1229/wrf/p1229-${case}/wrfout"
endif
#setenv SAVED "$rundir/saved"
setenv TMPTRN $rundir
if ($?CYCLE == 0) then
#   setenv CYCLE 3
   setenv CYCLE 6
endif
if ($?FCSTLEN == 0) then
#   setenv FCSTLEN 6
   setenv FCSTLEN 30
endif

set day=`echo $iday | sed -e 's/^......//'`
mkdir $rundir/arl
if (-e $config/full_levels.txt) /bin/cp $config/full_levels.txt $rundir/arl
if (-e $config/half_levels.txt) /bin/cp $config/half_levels.txt $rundir/arl

foreach hr ( $hrs )
  set itime=${iday}${hr}
  $NWPDIR/p1229/wrfv2/run_oneday.pl -rundir $rundir -nodogrib -mv_host $mv_host -dowrf -itime $itime -fcstlen $FCSTLEN \
   -ndomains $ndomains\
   -qsub "$qsub" -peqsub "$peqsub" -sge $sge -nproc $nproc\
   -config=$config -oldrun=$oldrun -siwrf=$siwrf -wrfwrf=$wrfwrf -$debug
  $NWPDIR/p1229/wrfv2/postp_oneday.csh $ndomains $itime
end

setenv RUNDIR $run_dir
if ($?ARCHIVE) then
   if ($ARCHIVE == "none") exit
else
   setenv ARCHIVE "/project/p1229/wrf/p1229-${case}"
endif
if ($?TASKS) then
   if ($TASKS == "none") exit
else
   setenv TASKS "wrf arl"
endif
$NWPDIR/p1229/wrfv2/archive_oneday.csh ${iday} ${eday} ${case} ${ndomains} $TASKS
