#!/usr/bin/perl

$|=1;

use strict;
use Time::Local;
use Getopt::Long ;
Getopt::Long::Configure('ignore_case') ;

# to be supplied via args:
my (@domains,$inittime,$interval,$fcstlen,$verbose,$debug,$tmpdir,$NWPDIR);

$interval=10800;
$fcstlen=30;
$verbose=1;
$debug=0;
$tmpdir=`pwd`;
chomp $tmpdir;
$NWPDIR="$ENV{HOME}/NWP";

my %options = ('verbose!' => \$verbose,       
	       'debug!' => \$debug,       
	       'interval=i' => \$interval,
	       'inittime=i' => \$inittime,
	       'fcstlen=i' => \$fcstlen,
	       'domains=i@' => \@domains,
	       'NWPDIR=s' => \$NWPDIR,
	       'tmpdir=s' => \$tmpdir);
my %helpopt = ('verbose!' => "turn on diagnostic output",
	       'debug!'=> "turn on debugging",
	       'inittime=i' => "REQ: initial time of forecast (yyyymmddhh)",
	       'fcstlen=i' => "length of forecast (hours)",
	       'domains=i@' => "Domain numbers to process",
	       'interval=is' => "output interval (sec)",
	       'NWPDIR=s' => "parent directory of native cvs working directories",
	       'tmpdir=s' => "Working directory for this run");

#my $help=1 unless GetOptions (%options);
my $help;
GetOptions (%options);
$help = 1 if (!$inittime);
$verbose = 1 if ($debug || $help);
@domains=(1,2) unless @domains;
my $hrint=$interval/3600;

my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}

&help(\%options, \%helpopt) if $help;
chdir $tmpdir or die "Cannot chdir $tmpdir\n";
$ENV{NWPDIR} = "$NWPDIR";

# Loop over all domains

foreach my $domain ( @domains ) {
  my $tmproot="stats_narr.".$$.".d$domain";
  my $tmpfile=$tmproot.".r";

# Create R source code file, using inittime, domain, fcstlen, and hrint (from interval):

  open(TMPL,">$tmpfile") or die ("Cannot open >$tmpfile\n");
  print TMPL "# R code for stats_narr
nwpdir <- Sys.getenv('NWPDIR')
if (nchar(nwpdir) == 0) {
homedir <- Sys.getenv('HOME')
nwpdir <- paste(homedir,'NWP',sep='/')
}
source(paste(nwpdir,'generic/R/src/sourcedir.r',sep='/'))
sourcedir(paste(nwpdir,'generic/R/src',sep='/'))
sourcedir(paste(nwpdir,'native_R',sep='/'))
sourcedir(paste(nwpdir,'p1229/wrfv2',sep='/'))
inittime <- '".$inittime."'
domain <- '".$domain."'
tst <- stats.narr.onedomain(inittime,domain,fcstlen=$fcstlen,interval=$hrint)
assignr('stats.narr.d".$domain.".".$inittime."',tst,printTF=TRUE)
";
  close(TMPL) or warn("Could not close >$tmpfile\n");

# Run R job with this source code

  my $cmd="$NWPDIR/generic/scripts/Rjob.csh $tmproot";
  print "At ",scalar localtime()," Running $cmd\n" if $verbose;
  die ("Error running $cmd\n") if (system($cmd));
}

1;

sub help{
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 This script computes statistics in R from native files from narr and wrfout files:

 Options (REQ denotes required arguments):
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
