stats.narr.onedomain <- function(inittime,domain,fcstlen=30,interval=3,
                                 fhrs=util.i2c(seq(0,fcstlen,interval),2),
                                 indir='./',
                                 windows=NULL,lon.wrf=177,lat.wrf=176,psfc.narr=1,
                                 types=c('3d','10m','2m'),masks=c('none','psfc'),
                                 big.vmiss=1e20,small.vmiss=-1e20,iprint=2) {
  if (is.null(windows)) {
    windows <- array(NA,dim=c(3,4),dimnames=list(c('all','NE','d1d2'),
                                     c('lon.1','lon.2','lat.1','lat.2')))
    windows['all',] <- NA
    windows['NE',] <- c(-85,-55,35,65)
    windows['d1d2',] <- c(-78,-63,42,52)
  }
    
  if (substring(indir,nchar(indir),nchar(indir)) != '/') indir <- paste(indir,'/',sep='')
  narr.pre <- vector('list',length(types))
  names(narr.pre) <- types
  wrf.pre <- narr.pre
  narr.post <- narr.pre
  wrf.post <- narr.pre
  for (type in types) {
    found.type <- FALSE
    if (type == '3d') {
      narr.pre[[type]] <- paste(inittime,'/narr_',inittime,'_',sep='')
      narr.post[[type]] <- '00_all_100_all_3d'
      wrf.pre[[type]] <- paste(inittime,'/d',domain,'_',inittime,'_',sep='')
      wrf.post[[type]] <- '00_all_100_all_3d_narr_rot_3d'
      found.type <- TRUE
    }
    if (type == '10m') {
      narr.pre[[type]] <- paste(inittime,'/narr_',inittime,'_',sep='')
      narr.post[[type]] <- '00_uv_105_10_2d'
      wrf.pre[[type]] <- paste(inittime,'/d',domain,'_',inittime,'_',sep='')
      wrf.post[[type]] <- '00_uv_105_10_2d_narr_rot_2d'
      found.type <- TRUE
    }
    if (type == '2m') {
      narr.pre[[type]] <- paste(inittime,'/narr_',inittime,'_',sep='')
      narr.post[[type]] <- '00_all_105_2_norot'
      wrf.pre[[type]] <- paste(inittime,'/d',domain,'_',inittime,'_',sep='')
      wrf.post[[type]] <- '00_all_105_2_norot_narr_norot'
      found.type <- TRUE
    }
    if (!found.type) {
      warning(paste('Unsupported type=',type,'removed from output'))
      wrf.pre[[type]] <- NULL
      wrf.post[[type]] <- NULL
      narr.pre[[type]] <- NULL
      narr.post[[type]] <- NULL
      types <- types[types != type]
    }
  }

  out.list <- vector('list',length(types)+1)
  names(out.list) <- c('windows',types)
  out.list$windows <- windows
  
  for (fhr in fhrs) {
    if (iprint >= 1) cat(date(),': Processing fhr=',fhr,'\n');
    sfc.name <- paste(indir,inittime,'/narr_',inittime,'_',fhr,'00_all_1_all_norot',sep='')
    narr.sfc <- get.saved.grid(paste(sfc.name,'hdr',sep='.'),paste(sfc.name,'dat',sep='.'),
                               big.vmiss=big.vmiss,small.vmiss=small.vmiss)
    sfc.name <- paste(indir,inittime,'/d',domain,'_',inittime,'_',fhr,'00_all_1_all_norot_narr_norot',sep='')
    wrf.sfc <- get.saved.grid(paste(sfc.name,'hdr',sep='.'),paste(sfc.name,'dat',sep='.'),
                               big.vmiss=big.vmiss,small.vmiss=small.vmiss)
    skip <- is.null(narr.sfc$data) || is.null(wrf.sfc$data)
    for (type in types) {
      if (iprint >= 2) cat(' ',date(),': Processing type=',type,'\n');
      narr.name <- paste(indir,narr.pre[[type]],fhr,narr.post[[type]],sep='')
      narr.grid <- get.saved.grid(paste(narr.name,'hdr',sep='.'),paste(narr.name,'dat',sep='.'),
                               big.vmiss=big.vmiss,small.vmiss=small.vmiss)
      wrf.name <- paste(indir,wrf.pre[[type]],fhr,wrf.post[[type]],sep='')
      wrf.grid <- get.saved.grid(paste(wrf.name,'hdr',sep='.'),paste(wrf.name,'dat',sep='.'),
                               big.vmiss=big.vmiss,small.vmiss=small.vmiss)
      skip <- skip || is.null(narr.grid$data) || is.null(wrf.grid$data)
      if (skip) {
        cat('Skipping fhr=',fhr,'\n')
      } else {
        tmp.masks <- masks
        if (type != '3d') tmp.masks <- 'none'
        for (window in dimnames(windows)[[1]]) {
          if (iprint >= 3) cat(' ',' ',date(),': Processing window=',window,'\n');
          for (mask in tmp.masks) {
            if (iprint >= 4) cat(' ',' ',' ',date(),': Processing mask=',mask,'\n');
            tmp.narr <- stats.maskout(narr.grid,lonlat=windows[window,],mask,
                                lon=wrf.sfc$data[,,1,as.character(lon.wrf)],
                                lat=wrf.sfc$data[,,1,as.character(lat.wrf)],
                                psfc=narr.sfc$data[,,1,as.character(psfc.narr)])
            tmp.wrf <- stats.maskout(wrf.grid,lonlat=windows[window,],mask,
                                lon=wrf.sfc$data[,,1,as.character(lon.wrf)],
                                lat=wrf.sfc$data[,,1,as.character(lat.wrf)],
                                psfc=narr.sfc$data[,,1,as.character(psfc.narr)])
            tmp.stats <- stats.native(grid.1=tmp.wrf,grid.2=tmp.narr)
            if (is.null(out.list[[type]])) {
# Initialize output array for this type
              stats.names <- c('n','mean','rms','n12','bias','rmse','bias.norm','rmse.norm')
              out.list[[type]] <- array(NA,dim=c(dim(tmp.stats$n)[1:2],length(stats.names),
                                             length(fhrs),dim(windows)[1],length(tmp.masks)),
                                        dimnames=list(dimnames(tmp.stats$n)[[1]],
                                          dimnames(tmp.stats$n)[[2]],stats.names,fhrs,
                                          dimnames(windows)[[1]],tmp.masks))
            }
            for (stats.name in stats.names) {
              out.list[[type]][,,stats.name,fhr,window,mask] <- tmp.stats[[stats.name]]
            }
          } #endloop masks
        } #endloop windows
      } #endif skip
    } #endloop types
  } #endloop fhrs
  out.list
}

stats.maskout <- function(grid,lonlat,mask='none',lon=NULL,lat=NULL,psfc=NULL) {

  #mask out 4d-array (i,j,lev,var) according to:

  if (all(!is.na(lonlat))) {

    #keep only gridpoints inside lat-lon window (not done if any lon/lat limits are NA)

    if (is.null(lon) || is.null(lat)) stop('Cannot mask lonlat with missing lon,lat')
    #remap all lons to [-180,180]:
    lon <- ifelse(lon > 180,lon-360,lon)
    lonlat[c('lon.1','lon.2')] <- ifelse(lonlat[c('lon.1','lon.2')] > 180,
                                         lonlat[c('lon.1','lon.2')] - 360,
                                         lonlat[c('lon.1','lon.2')])
    if (lonlat['lon.1'] < lonlat['lon.2']) {
      ll.mask <- !is.na(lon) & lon >= lonlat['lon.1'] & lon <= lonlat['lon.2']
    } else {
# for longitude limits crossing the dateline (e.g., c(170,-170) for lons within 10deg of 180):
      ll.mask <- !is.na(lon) & lon >= lonlat['lon.1'] | lon <= lonlat['lon.2']
    }
    ll.mask <- ll.mask & lat >= lonlat['lat.1'] & lat <= lonlat['lat.2']
    ll.mask <- !ll.mask
    full.mask <- array(rep(ll.mask,prod(dim(grid$data)[3:4])),dim=dim(grid$data))
    grid$data[full.mask] <- NA
  }
  if(mask == 'psfc') {

    #keep only gridpoints (by level) above ground (mand p-level above psfc)

    if (is.null(psfc)) stop('Cannot mask psfc with missing psfc')
    levsPa <- 100*as.numeric(dimnames(grid$data)[[3]])
    for (k in 1:dim(grid$data)[3]) {
      p.mask <- is.na(psfc) | psfc < levsPa[k]
      for (var in 1:dim(grid$data)[4]) {
        slice <- grid$data[,,k,var]
        slice[p.mask] <- NA
        grid$data[,,k,var] <- slice
      }
    }
  }
  grid
}

narr.stats.combine <- function(in.list=NULL,in.dir=NULL,domain=NULL,
                               pattern=NULL,full=FALSE) {
  if (is.null(in.list)) {
    if (is.null(pattern) && !is.null(domain))
      pattern <- paste('.RDatastats.narr.d',domain,sep='')
    if (is.null(in.dir) || is.null(pattern))
      stop('Need to provide in.list or in.dir and domain or pattern')
    in.files <- list.files(in.dir,pattern=pattern,all.files=TRUE)
    if (length(in.files) < 1)
      stop (paste('No matching files found in',in.dir,'for pattern=',pattern))
    in.names <- substring(in.files,7)
    in.list <- vector('list',length(in.names))
    names(in.list) <- in.names
    for (i in 1:length(in.names)) in.list[[i]] <- getr(in.names[i],in.dir)
  }
  out.list <- list()
  for (ilist in 1:length(in.list)) {
    tmp <- in.list[[ilist]]
    if (ilist == 1) {
      out.list <- vector('list',length(tmp))
      names(out.list) <- names(tmp)
      if (!is.null(tmp$windows)) out.list$windows <- tmp$windows
    }
    for (type in names(out.list)[names(out.list) != 'windows']) {
      if (ilist == 1) {
        tmp.dim <- dim(tmp[[type]])
        tmp.names <- dimnames(tmp[[type]])
        out.dim <- c(tmp.dim,length(in.list))
        out.names <- tmp.names
        out.names[[length(out.dim)]] <- names(in.list)
        out.list[[type]] <- array(NA,dim=out.dim,dimnames=out.names)
      }
      if (length(out.dim) != 7)
        stop(paste('Unexpected dim of output: ',paste(out.dim,collapse=' ')))
      out.list[[type]][,,,,,,ilist] <- tmp[[type]]
    }
  }
  if (full) out.list <- list(out=out.list,input=in.list)
  out.list
}

plot.cmb.stats <- function(stats,type='3d',lev='500',var='7',region='NE',name=NULL,
                           main=paste(name,region,lev,var,sep='.'),psname=paste(main,'ps',sep='.'),
                           ylab=switch(var,'7'='m','11'='K','33'='m/s','34'='m/s','52'='Pct'),
                           ylim=NULL,xlim=NULL,add=FALSE,mask=NULL) {

  if (is.null(mask)) mask <- switch(type,'3d'='psfc','none')
  if (!is.null(psname)) postscript(psname,horizontal=F,paper='letter')
  if (is.null(ylim)) ylim <- range(stats[[type]][lev,var,c('bias','rmse'),,,mask,],na.rm=TRUE)
  if (is.null(xlim)) xlim <- range(as.numeric(dimnames(stats[[type]])[[4]]),na.rm=TRUE)
  if (!add) plot(xlim,ylim,type='n',xlab='Fcst Hr',ylab=ylab,main=main)
  for (i in 1:dim(stats[[type]])[7]) lines(as.numeric(dimnames(stats[[type]])[[4]]),
                                           stats[[type]][lev,var,'rmse',,region,mask,i],col=i)
  points(as.numeric(dimnames(stats[[type]])[[4]]),
         apply(stats[[type]][lev,var,'rmse',,region,mask,],MAR=1,FUN=mean))
  for (i in 1:dim(stats[[type]])[7]) lines(as.numeric(dimnames(stats[[type]])[[4]]),
                                           stats[[type]][lev,var,'bias',,region,mask,i],col=i,lty=2)
  points(as.numeric(dimnames(stats[[type]])[[4]]),
         apply(stats[[type]][lev,var,'bias',,region,mask,],MAR=1,FUN=mean))
  lines(range(as.numeric(dimnames(stats[[type]])[[4]])),c(0,0))
  if (!is.null(psname)) {
    dev.off()
    cat("Generated",psname,"\n")
  }
}
