#!/usr/bin/perl
# $Id: verify_fcst.pl,v 1.2 2006/01/16 15:40:33 trn Exp $

# script for performing verification against preprocessed analyses (real input files)
# of one WRF forecast
# WRF output files are in GRIB (get converted to native)
# REAL input files are in netcdf (get converted to native)

use strict;
use File::Basename;
use Getopt::Long ;

umask oct("002");
$|=1; #use unbuffered I/O

my %filesuf=('half' => 'half_3d','full' => 'full_3d','u' => 'u_3d','v' => 'v_3d',
	     'sfc' => 'sfc_2d','pcp' => 'pcp_2d', '2m' => '2m_2d', 'uv10m' => 'uv10m_2d', 
	     'soil' => 'soil_3d');
my @exts=('.hdr','.dat');

# Makes some assumptions about directories under HOME
my $home=$ENV{HOME};
my $wrfutil="$home/NWP/WRF_util/inhouse_util";
my $nativeutil="$home/NWP/native_util";
# for nativestats.csh:
$ENV{S_DIR}="$home/MySwork";
my $nativestats="$home/NWP/generic/splus/src/nativestats.csh";

my ($verbose, $debug, $help, $real_ncdf, $real_native, $wrf_grib, $wrf_native, @types, @dates, $ihr, @hrs, $solhost);
$verbose=1;

my %options = (
	       "verbose!"        => \$verbose, 
	       "debug!"        => \$debug, 
	       "help|?!"         => \$help,
	       "real_native=s"  => \$real_native,
	       "wrf_native=s"  => \$wrf_native,
	       "real_ncdf=s"  => \$real_ncdf,
	       "wrf_grib=s"  => \$wrf_grib,
	       "types=s@"  => \@types,
	       "dates=i@"  => \@dates,
	       "ihr=s"  => \$ihr,
	       "solhost=s"  => \$solhost,
	       "hrs=s@"  => \@hrs,
              );
my %helpopt = (
	       "verbose!"        => "printout control", 
	       "debug!"        => "debugging control", 
	       "help|?!"         => "generate help message",
	       "real_native=s"  => "directory containing real_native files",
	       "wrf_native=s"  => "directory containing wrf_native files",
	       "real_ncdf=s"  => "directory containing real_ncdf files",
	       "wrf_grib=s"  => "directory containing wrf_grib files",
	       "types=s@"  => "types to process",
	       "dates=i@"  => "[REQ:]  Starting date (yyyymmdd) of fcst; if it extends into following
days, then also provide yyyymmdd for day+1, day+2, ...",
	       "ihr=s"  => "Starting hour (2-digit)",
	       "hrs=s@"  => "Fcst hours (2-digit)",
	       "solhost=s"  => "host name for machine running Solaris and Splus",
	      );

$help=1 unless GetOptions (%options);
$help=1 unless @dates; #required
$wrf_native="$home/junk/native" unless $wrf_native; #optional
$real_native="$home/junk/native/wrf_real" unless $real_native; #optional
$wrf_grib="/beehive/scratch/day14/trn/saved" unless $wrf_grib;
$real_ncdf="/beehive/scratch/day14/trn/saved/wrf_real_input" unless $real_ncdf;
$solhost="burst" unless $solhost;
@hrs=("00","06","12","18","24","30") unless @hrs;

@types = ('half', 'u', 'v') unless @types; #default
my @call = caller(0);
if (@call > 6) {
    $0 = $call[6] if ($call[3] eq "(eval)");
}
$verbose=1 if $help;
$verbose=1 if $debug;
if ($verbose) {
  print "At ",scalar(localtime)," running $0 with:\n";
  foreach my $key (keys(%options)) {
    if (substr($key,length($key)-1) eq "@") {
      print "  $key = @{$options{$key}}\n";
    } else {
      print "  $key = ${$options{$key}}\n";
    }
  }
}
&help(\%options, \%helpopt) if $help;

my $cmd0=dirname($0);

my $cmd;
my $rundir=`pwd`;
chomp $rundir;

foreach my $test_dir ($wrf_native,$real_native,"$wrf_native/$dates[0]") {
  if (!chdir($test_dir)) {
    unlink $test_dir if (-e $test_dir);
    mkdir ($test_dir) or die "Cannot mkdir $test_dir\n";
  }
}

foreach my $fhr (@hrs) {
  chdir ($rundir) or die "Cannot chdir to rundir=$rundir\n";
  my $idate=0;
  my $vhr=$ihr+$fhr;
  while ($vhr > 23) {
    $idate=$idate+1;
    $vhr=$vhr-24;
  }
  my $vdate=$dates[$idate];
  while (length($vhr) < 2) {
    $vhr="0".$vhr;
  }
  my $vyr=substr($vdate,0,4);
  my $vmm=substr($vdate,4,2);
  my $vda=substr($vdate,6,2);

  print "fhr= ".$fhr." vdate= ".$vdate." vhr= ".$vhr."\n";

# look for native file(s), generate if needed
  my ($do_wrf, $do_real);
  my $this_wrfgrib=$wrf_grib."/".$dates[0]."/wrfout_d01_".$vyr."-".$vmm."-".$vda."_".$vhr.":00:00";
  my $this_wrfnative=$wrf_native."/".$dates[0]."/d01_".$fhr."hr";
  my $this_realncdf=$real_ncdf."/wrf_real_input_em.d01.".$vyr."-".$vmm."-".$vda."_".$vhr.":00:00";
  my $this_realnative=$real_native."/real_native_".$vyr."-".$vmm."-".$vda."_".$vhr;
  my ($type, $ext);
  for $type (@types) { 
    for $ext (@exts) {
      $do_wrf=1 if (! -e $this_wrfnative."_".$filesuf{$type}.$ext);
      print "do_wrf=$do_wrf for $this_wrfnative"."_".$filesuf{$type}.$ext."\n" if $debug;
      $do_real=1 if (! -e $this_realnative."_".$filesuf{$type}.$ext);
      print "do_real=$do_real for $this_realnative"."_".$filesuf{$type}.$ext."\n" if $debug;
    }
  }

  if ($do_wrf) {
    my ($this_file, $this_dir, $this_suf) = fileparse($this_wrfnative,'');
    if (!chdir ($this_dir)) {
      warn "Could not chdir $this_dir \n";
    } else {
      $cmd="$nativeutil/wrf2native.pl -gribfile $this_wrfgrib -nativefile $this_file";
      $cmd="$cmd -parmfile parm_wrfgrib_stilt";
      print "At ",scalar(localtime)," running $cmd \n" if $verbose;
      warn "Error running $cmd\n" if system($cmd);
    }
  }
  if ($do_real) {
    my ($this_file, $this_dir, $this_suf) = fileparse($this_realnative,'');
    if (!chdir ($this_dir)) {
      warn "Could not chdir $this_dir \n";
    } else {
      $cmd="$wrfutil/ncwrf_to_native.pl -ncwrf $this_realncdf -wrf_native $this_wrfnative";
      $cmd="$cmd -native $this_file";
      print "At ",scalar(localtime)," running $cmd \n" if $verbose;
      warn "Error running $cmd\n" if system($cmd);
    }
  }

  chdir ($rundir) or die "Cannot chdir to rundir=$rundir\n";
  for my $type (@types) {
    $cmd="rsh -n $solhost "."'("."cd $rundir; $nativestats ".$this_wrfnative."_".$filesuf{$type}." ".$this_realnative."_".$filesuf{$type};
    $cmd="$cmd debug" if $debug;
    $cmd=$cmd.")'";
    print "At ",scalar(localtime)," running $cmd \n" if $verbose;
    warn "Error running $cmd\n" if system($cmd);
    my $outstats=basename($this_wrfnative)."_".$filesuf{$type}.".".basename($this_realnative)."_".$filesuf{$type}.".stats";
    my $newstats=basename($this_wrfnative)."_".$filesuf{$type};
    $newstats =~ tr/_-/../;
    $cmd="/bin/cp $outstats $dates[0]/$newstats";
    print "At ",scalar(localtime)," running $cmd \n" if $verbose;
    warn "Error running $cmd\n" if system($cmd);
  }
}
1;

sub help {
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 this script runs a forecast verification by computing statistics of differences
 between a WRF forecast output (in GRIB, converted to native if needed) and
 wrf_real_input_em files (in netcdf, converted to native if needed).
 Splus is used to compute difference stats
 Options:
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options]\n";
}
