#!/usr/bin/perl

$|=1;

use strict;

use Getopt::Long ;
Getopt::Long::Configure('ignore_case') ;
use File::Basename;

# Cover script for wrf-narr verification
# All arguments (including options) other than the 3 captured here are passed on to the lower-level scripts

# to be supplied via args:
my ($dopostp,$dostats,$dograds,$inittime,$clean,$allclean);

$dopostp=1;
$dostats=1;
$dograds=1;
$clean=1;
$allclean=0;

my %options = ('dopostp!' => \$dopostp,
	       'dostats!' => \$dostats,
	       'dograds!' => \$dograds,
	       'inittime=i' => \$inittime,
	       'clean!' => \$clean,
	       'allclean!' => \$allclean);
my %helpopt = ('dopostp!' => "run postp_narr.pl",
	       'dostats!' => "run stats_narr.pl",
	       'inittime=i' => "REQ: initial time of forecast (yyyymmddhh)",
	       'clean!' => "clean out wrfpostp PRESS-GRIB after postp",
	       'allclean!' => "clean out entire inittime directory and all matching narr files",
	       'dograds!' => "run grads_narr.pl");

my $help=1 unless GetOptions (%options);
$help=1 unless $inittime;
my @args=@ARGV;
$clean = 1 if $allclean;

my $cmd0 = $0;
my @call = caller(0);
if (@call > 6) {
    $cmd0 = $call[6] if ($call[3] eq "(eval)");
}
my $pwd0 = dirname($cmd0);

print "At ",scalar(localtime)," running $0 with:\n";
foreach my $key (keys(%options)) {
  if (substr($key,length($key)-1) eq "@") {
    print "  $key = @{$options{$key}}\n";
  } else {
    print "  $key = ${$options{$key}}\n";
  }
}
print " and args=@args\n";
&help(\%options, \%helpopt) if $help;

my $cmd;
if ($dopostp) {
  $cmd="$pwd0/postp_narr.pl -inittime $inittime @args";
  print "At ",scalar localtime()," Running $cmd\n";
  die ("Error running $cmd\n") if (system($cmd));
}

system("/bin/rm -fr $inittime/PRESS-GRIB") if ($clean);

if ($dostats) {
  $cmd="$pwd0/stats_narr.pl -inittime $inittime @args";
  print "At ",scalar localtime()," Running $cmd\n";
  die ("Error running $cmd\n") if (system($cmd));
}

if ($dograds) {
  $cmd="$pwd0/grads_narr.pl -inittime $inittime @args";
  print "At ",scalar localtime()," Running $cmd\n";
  die ("Error running $cmd\n") if (system($cmd));
}

system("/bin/rm -fr $inittime") if ($allclean);
my $initday=substr($inittime,0,8);
system("/bin/rm -fr narr/*${initday}*") if ($allclean);
system("/bin/rm -fr stageiv/*${initday}*") if ($allclean);

1;



sub help{
  my %options=%{$_[0]};
  my %helpopt=%{$_[1]};
  print "
Help for $0 
 This script runs the low-level scripts for wrf-narr verification
 Options used by this top-level script (REQ denotes required arguments):
   all others (preceded by --): passed on to low-level scripts
";
foreach my $key (keys(%options)) {
  print "  $key : $helpopt{$key}\n";
}
die 
"Usage: $0 [options] [-- more options]\n";
}
