      SUBROUTINE PROCESS
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .     
C SUBPROGRAM:    PROCESS     DRIVER FOR MAJOR POST ROUTINES.
C   PRGRMMR: TREADON         ORG: W/NP2      DATE: 92-12-21       
C     
C ABSTRACT:
C     THIS ROUTINE CALLS THE MAJOR POST PROCESSOR ROUTINES.
C     THESE ROUTINES ARE
C        MDLFLD  - CALCULATE NMC SLP, SET BELOW SURFACE FIELDS,
C                  AND POSTS DATA ON MODEL SURFACES.
C        MDL2P   - POSTS DATA ON ISOBARIC SURFACES.
C        SURFCE  - POSTS SOUNDING DATA, SURFACE BASED FIELDS, 
C                  AND STATIC OR FIXED FIELDS.
C        CLDRAD  - POST SOUNDING/CLOUD/RADIATION FIELDS.
C        MISCLN  - POST MISCELLANEOUS (SPECIAL) FIELDS.
C        FIXED   - POST FIXED FIELDS.
C   .     
C     
C PROGRAM HISTORY LOG:
C   92-12-21  RUSS TREADON
C   98-06-01  T BLACK - CONVERSION OF POST FROM 1-D TO 2-D
C   00-01-05  JIM TUCCILLO - MPI VERSION
C   01-10-25  H CHUANG - MODIFIED TO PROCESS HYBRID MODEL OUTPUT
C   02-06-19  MIKE BALDWIN - WRF VERSION
C     
C USAGE:    CALL PROCESS
C   INPUT ARGUMENT LIST:
C     NONE     
C
C   OUTPUT ARGUMENT LIST: 
C     NONE
C     
C   OUTPUT FILES:
C     NONE
C     
C   SUBPROGRAMS CALLED:
C     UTILITIES:
C       MDLFLD   - POST DATA MDL SURFACES.
C       MDL2P    - POST DATA ON PRESSURE SURFACES.
C       SURFCE   - POST SURFACE BASED FIELDS.
C       CLDRAD   - POST SOUNDING/CLOUD/RADIATION FIELDS.
C       MISCLN   - POST MISCELLANEOUS FIELDS.
C       FIXED    - POST FIXED FIELDS.
C     LIBRARY:
C       COMMON   - OUTGRD
C     
C   ATTRIBUTES:
C     LANGUAGE: FORTRAN
C     MACHINE : CRAY C-90
C$$$  
C
C     
C     INCLUDE PARAMETERS.
!      INCLUDE "parmeta"
!      INCLUDE "parmout"
C
C     INCLUDE I/O COMMON BLOCK.
      INCLUDE "CTLBLK.comm"
C
C     DECLARE VARIABLES.
C     
      CHARACTER*6  DATSET,PROJ
      LOGICAL NORTH,SIGMA
c
      common/tim_info/ETAFLD2_tim,ETA2P_tim,SURFCE2_tim, CLDRAD_tim,
     *                MISCLN_tim,FIXED_tim
C
C****************************************************************************
C     START SUBROUTINE PROCESS.
C
C     
C     COMPUTE/POST FIELDS ON MDL SURFACES.
C
      btim = timef()
      CALL MDLFLD
      ETAFLD2_tim = ETAFLD2_tim +(timef() - btim)
      print*,'coming back from MDLFLD in PROCESS'
C
C     COMPUTE/POST FIELDS ON PRESSURE SURFACES.
      btim = timef()
      CALL MDL2P
      ETA2P_tim = ETA2P_tim +(timef() - btim)
C
C     COMPUTE/POST FIELDS ON SIGMA SURFACES.
      btim = timef()
      CALL MDL2SIGMA
      MDL2SIGMA_tim = MDL2SIGMA_tim +(timef() - btim)
C
      CALL MDL2AGL
C
C     COMPUTE/POST SURFACE RELATED FIELDS.
      btim = timef()
      CALL SURFCE
      SURFCE2_tim = SURFCE2_tim +(timef() - btim)
C
C     COMPUTE/POST SOUNDING AND CLOUD RELATED FIELDS.
      btim = timef()
      CALL CLDRAD
      CLDRAD_tim = CLDRAD_tim +(timef() - btim)
C
C     COMPUTE/POST TROPOPAUSE DATA, FD LEVEL FIELDS,
C     FREEZING LEVEL HEIGHT AND RH, BOUNDARY LAYER FIELDS,
C     AND LFM-NGM LOOK-ALIKE FIELDS.
      btim = timef()
      CALL MISCLN
      MISCLN_tim = MISCLN_tim +(timef() - btim)
C
C     POST FIXED FIELDS.
      btim = timef()
      CALL FIXED
      FIXED_tim =  FIXED_tim +(timef() - btim)
C     
C     END OF ROUTINE.
C     
      RETURN
      END
